// 
//  coOccurenceMatrix.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: coOccurenceMatrix.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _COOCCURENCEMATRIX_H_
#define _COOCCURENCEMATRIX_H_

#include "image.h"

template< PixelDepth D > class Image;

class CoOccurenceMatrix
{

    typedef unsigned int StorageType;

public:

    enum Orientation {
        Orient0,
        Orient45,
        Orient90,
        Orient135,
        NB_ORIENTATIONS
    };

    CoOccurenceMatrix(
        const Image< STUInt8 >* img,
        unsigned short int      distance
    );

    ~CoOccurenceMatrix();
    
    unsigned short int getDistance() const;
    
    StorageType getMatrixValue(
        const int orientation,
        const int ci,
        const int cj
    ) const;
    
    float getEnergy() const;
    float getEntropy() const;
    float getCorrelation() const;
    float getInverseDifferenceMoment() const;
    float getInertia() const;
    float getClusterShade() const;
    float getClusterProminence() const;
                               
private:

    void fill( const Image< STUInt8 >* img );
    void processDirections(
        const Image< STUInt8>* img,
        const int   w,
        const int   h,
        const int   x,
        const int   y
    );
    
    void fillTableMeans();
    
    void computeEnergy();
    void computeEntropy();
    void computeCorrelation();
    void computeInverseDifferenceMoment();
    void computeInertia();
    void computeClusterShade();
    void computeClusterProminence();
    
    float _energy;
    float _entropy;
    float _correlation;
    float _inverseDifferenceMoment;
    float _inertia;
    float _clusterShade;
    float _clusterProminence;

    const unsigned short int _distance;
    StorageType              _table[NB_ORIENTATIONS][256][256];
    float                    _tableMeans[256][256];
    unsigned int             _nbNeighbour[NB_ORIENTATIONS];

};

#include "coOccurenceMatrix.inline.h"

#endif /* _COOCCURENCEMATRIX_H_ */
