// 
//  coOccurenceMatrixFeature.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: coOccurenceMatrixFeature.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "coOccurenceMatrixFeature.h"
#include "coOccurenceMatrix.h"
#include "scoped_ptr.h"
#include <ostream>
#include <iterator>

///////////////////////////////////////////////////////////////////////////////

int CoOccurenceMatrixFeature::getNbStatistics()
{
    return 7;
}

///////////////////////////////////////////////////////////////////////////////

ImageFeature::Type CoOccurenceMatrixFeature::getType() const
{
    return CO_OCCURENCE;
}

///////////////////////////////////////////////////////////////////////////////

ColorSpaces::ColorSpace CoOccurenceMatrixFeature::getColorSpace() const
{
    return ColorSpaces::Grayscale;
}

///////////////////////////////////////////////////////////////////////////////

CoOccurenceMatrixFeature::CoOccurenceMatrixFeature(
    const Image< STUInt8 >* img,
    unsigned char           distances
)
{
    float energy = 0.0f;
    float entropy = 0.0f;
    float correlation = 0.0f;
    float inverseDiffMoment = 0.0f;
    float inertia = 0.0f;
    float clusterShade = 0.0f;
    float clusterProminence = 0.0f;

    int n = 0;
    for( int i = 0; i < 4; ++i) {
        if( distances & ( 1 << i ) ) {
            scoped_ptr< CoOccurenceMatrix > coOcc(
                new CoOccurenceMatrix( img, i + 1 ) );
            energy            += coOcc->getEnergy();
            entropy           += coOcc->getEntropy();
            correlation       += coOcc->getCorrelation();
            inverseDiffMoment += coOcc->getInverseDifferenceMoment();
            inertia           += coOcc->getInertia();
            clusterShade      += coOcc->getClusterShade();
            clusterProminence += coOcc->getClusterProminence();
            ++n;
        }
    }

    assert( n > 0 );

    addStatistic( energy / n );
    addStatistic( entropy / n );
    addStatistic( correlation / n );
    addStatistic( inverseDiffMoment / n );
    addStatistic( inertia / n );
    addStatistic( clusterShade / n );
    addStatistic( clusterProminence / n );
}

///////////////////////////////////////////////////////////////////////////////

CoOccurenceMatrixFeature::CoOccurenceMatrixFeature( std::istream& is )
: TextureFeature( is, getNbStatistics() )
{}
