// 
//  collectionReport.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: collectionReport.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _COLLECTIONREPORT_H_
#define _COLLECTIONREPORT_H_

#include "searchResult.h"
#include "imageRanker.h"
#include "imageReportPostProcessor.h"
#include "imageReport.h"
#include "imageFeature.h"
#include "colorSpaces.h"
#include <vector>
#include <set>
#include <iterator>

class CollectionReport : public ReportFileComponent
{

public:

    typedef shared_ptr< ImageReport > ReportPtr;

    struct LessThanImageReport {
        bool operator()( const ReportPtr& l, const ReportPtr& r ) const {
            return l->getFilename() < r->getFilename(); }
    };

    typedef std::set< ReportPtr, LessThanImageReport > ImageReportSet;
    typedef ImageReportSet::const_iterator ImageReportIterator;
    typedef std::string Filename;

    // byte 1: endianness ( 0 for little, 1 for big )
    // byte 2: nb. of features
    // followed by "nb" 16 byte descriptors
    class Header : public ReportFileComponent
    {

    public:

        typedef std::vector< ImageFeature::Descriptor > DescriptorVec;
        typedef std::pair< DescriptorVec::const_iterator, bool > FeaturePair;

        Header();
        Header( std::istream& is );

        void add( const ImageFeature::Descriptor& desc );

        FeaturePair getFeature(
            ImageFeature::Type,
            ColorSpaces::ColorSpace = ColorSpaces::UNKNOWN
        ) const;

        // See ReportFileComponent
        void write( std::ostream& os ) const;

    private:

        DescriptorVec _descriptors;

    };

    CollectionReport();
    ~CollectionReport();

    // See ReportFileComponent
    void write( std::ostream& os ) const;

    unsigned int getNbImages() const;
    bool insert( ReportPtr imgReport );

    // Iterates over the collection, calling operator() on "postProcessor"
    void postProcess( ImageReportPostProcessor& postProcessor );

    // Iterates over the collection, calling operator() on "ranker" and
    // saves the result paired with the ImageReport pointer in the container
    // specified through "resultIterator"
    void collect(
        std::multiset< SearchResult >& results,
        const ImageRanker&             ranker
    ) const;

private:

    ImageReportSet _reports;
};

#endif /* _COLLECTIONREPORT_H_ */
