// 
//  color.inline.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: color.inline.h 2926 2008-04-13 22:45:08Z benoit $
// 

#include <cstring>
#include <math.h>
#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Color< D >::Color(
    ColorSpaces::ColorSpace colorSpace,
    const StorageType* values
) : _colorSpace( colorSpace ),
    _values( new StorageType[ ColorSpaces::getNbChannels( colorSpace ) ] )
{
    memcpy( _values, values,
        sizeof( StorageType ) * ColorSpaces::getNbChannels( _colorSpace ) );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Color< D >::Color( const Color< D >& c )
: _colorSpace( c._colorSpace ),
  _values( new StorageType[ ColorSpaces::getNbChannels( c._colorSpace ) ] )
{
    memcpy( _values, c._values,
        sizeof( StorageType ) * ColorSpaces::getNbChannels( _colorSpace ) );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Color< D >& Color< D >::operator=( const Color< D >& c )
{
    if( _values ) {
        delete[] _values;
        _values = 0;
    }

    _colorSpace = c._colorSpace;

    _values = new StorageType[ ColorSpaces::getNbChannels( _colorSpace ) ];
    memcpy( _values, c._values,
        sizeof( StorageType ) * ColorSpaces::getNbChannels( _colorSpace ) );

    return *this;
}


///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Color< D >::~Color()
{
   delete[] _values;
   _values = 0;
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
ColorSpaces::ColorSpace Color< D >::getColorSpace() const
{
    return _colorSpace;
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
typename Color< D >::StorageType Color< D >::getValue(
    ColorSpaces::Channel channel
) const
{
    assert( ColorSpaces::isNative( _colorSpace, channel ) );
    return *( _values + ColorSpaces::getChannelIndex( _colorSpace, channel ) );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Color< D > operator-( const Color< D >&l, const Color< D >&r )
{
    assert( l._colorSpace == r._colorSpace );

    Color< D > diff( l );

    for( unsigned int i = 0;
         i < ColorSpaces::getNbChannels( l._colorSpace );
         ++i ) {
       diff._values[i] -= r._values[i];
    }

    return diff;
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
float Color< D >::norm() const
{
    float norm = 0.0f;
    for( unsigned int i = 0;
         i < ColorSpaces::getNbChannels( _colorSpace );
         ++i ) {
         norm += _values[i] *_values[i];
    }
    return sqrtf( norm );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
void Color< D >::write( std::ostream& os ) const
{
    // Dump the channel values to the stream
    os.write( reinterpret_cast< const char* >( _values ),
              sizeof( StorageType ) *
              ColorSpaces::getNbChannels( _colorSpace ) );
}
