// 
//  colorConverter.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: colorConverter.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "colorConverter.h"
#include "image.h"
#include <algorithm>

///////////////////////////////////////////////////////////////////////////////

ColorConverter::ColorConverter()
{}

///////////////////////////////////////////////////////////////////////////////

ColorConverter::~ColorConverter()
{}

///////////////////////////////////////////////////////////////////////////////

DirectColorConverter::DirectColorConverter()
{}

///////////////////////////////////////////////////////////////////////////////

DirectColorConverter::~DirectColorConverter()
{}

///////////////////////////////////////////////////////////////////////////////

ImageBase* DirectColorConverter::convert( const ImageBase* srcImg ) const
{
    ImageBase* outImg = ImageBase::create(
       srcImg->getWidth(),
       srcImg->getHeight(),
       getTargetColorSpace(),
       getTargetDepth()
    );

   convert( srcImg->getPixelBuffer(), outImg->getPixelBuffer() );

   return outImg;
}

///////////////////////////////////////////////////////////////////////////////

CompositeColorConverter::CompositeColorConverter()
{}

///////////////////////////////////////////////////////////////////////////////

CompositeColorConverter::~CompositeColorConverter()
{}

///////////////////////////////////////////////////////////////////////////////

ColorSpaces::ColorSpace CompositeColorConverter::getSourceColorSpace() const
{
    assert( !_chain.empty() );
    return (*_chain.begin())->getSourceColorSpace();
}

///////////////////////////////////////////////////////////////////////////////

PixelDepth CompositeColorConverter::getSourceDepth() const
{
    assert( !_chain.empty() );
    return (*_chain.begin())->getSourceDepth();
}

///////////////////////////////////////////////////////////////////////////////

ColorSpaces::ColorSpace CompositeColorConverter::getTargetColorSpace() const
{
    assert( !_chain.empty() );
    return (*(_chain.end() - 1))->getTargetColorSpace();
}

///////////////////////////////////////////////////////////////////////////////

PixelDepth CompositeColorConverter::getTargetDepth() const
{
    assert( !_chain.empty() );
    return (*(_chain.end() - 1))->getTargetDepth();
}

///////////////////////////////////////////////////////////////////////////////

void CompositeColorConverter::linkConverter( DirectColorConverter* converter )
{
    assert( converter != 0 );

    // Make sure the sockets fit
    if( !_chain.empty() ) {
        assert( (*(_chain.end()-1))->getTargetColorSpace() ==
                converter->getSourceColorSpace() );
        assert( (*(_chain.end()-1))->getTargetDepth() ==
                converter->getSourceDepth() );
    }

    _chain.push_back( converter );
}

///////////////////////////////////////////////////////////////////////////////

ImageBase* CompositeColorConverter::convert( const ImageBase* srcImg ) const
{
    assert( !_chain.empty() );
    assert( _chain.size() >= 2 );

    const unsigned int width = srcImg->getWidth();
    const unsigned int height = srcImg->getHeight();

    ConvertChain::const_iterator it = _chain.begin();

    ImageBase* img_a;
    ImageBase* img_b;

    img_a = ImageBase::create(
       width,
       height,
       (*it)->getTargetColorSpace(),
       (*it)->getTargetDepth()
    );
    (*it)->convert( srcImg->getPixelBuffer(), img_a->getPixelBuffer() );

    while( ++it != _chain.end() ) {
        img_b = ImageBase::create(
            width,
            height,
           (*it)->getTargetColorSpace(),
           (*it)->getTargetDepth()
        );
        
       (*it)->convert( img_a->getPixelBuffer(), img_b->getPixelBuffer() );

       delete img_a;

       std::swap( img_a, img_b );
    }

    return img_a;
}
