// 
//  colorConverter.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: colorConverter.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _COLORCONVERTER_H_
#define _COLORCONVERTER_H_

#include "pixelBuffer.h"
#include "colorSpaces.h"
#include "pixelDepths.h"

class ImageBase;

class ColorConverter : public ColorSpaces
{

public:

    virtual ~ColorConverter();

    virtual ImageBase* convert(
        const ImageBase* srcImg
    ) const = 0;

    virtual ColorSpace getSourceColorSpace() const = 0;
    virtual PixelDepth getSourceDepth() const = 0;
    virtual ColorSpace getTargetColorSpace() const = 0;
    virtual PixelDepth getTargetDepth() const = 0;

protected:

    ColorConverter();

private:
    
    ColorConverter( const ColorConverter& );
};

class DirectColorConverter : public ColorConverter
{

public:

    virtual ~DirectColorConverter();

    ImageBase* convert(
       const ImageBase* srcImg
    ) const;

    virtual void convert(
        const PixelBuffer& srcBuffer,
        PixelBuffer&       outBuffer
    ) const = 0;

protected:

    DirectColorConverter();

private:
    
    DirectColorConverter( const ColorConverter& );
};

class CompositeColorConverter : public ColorConverter
{

    typedef std::vector< DirectColorConverter* > ConvertChain;

public:

    CompositeColorConverter();
    ~CompositeColorConverter();

    ColorSpace getSourceColorSpace() const;
    PixelDepth getSourceDepth() const;
    ColorSpace getTargetColorSpace() const;
    PixelDepth getTargetDepth() const;
    
    virtual ImageBase* convert(
        const ImageBase* srcImg
    ) const;

    void linkConverter( DirectColorConverter* converter );

private:

    ConvertChain _chain;
};

#endif /* _COLORCONVERTER_H_ */
