// 
//  colorSpaces.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: colorSpaces.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "colorConverter.h"
#include "colorSpaces.h"
#include "image.h"

#include <limits>
#include <vector>
#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

bool ColorSpaces::_isNative[NB_COLOR_SPACES][NB_CHANNELS];
ColorSpaces::ColorSpaceInfo ColorSpaces::_colorSpaceInfo[NB_COLOR_SPACES];
bool ColorSpaces::_initialized( false );

///////////////////////////////////////////////////////////////////////////////

void ColorSpaces::registerColorSpace(
    const ColorSpace              colorSpace,
    const std::string&            name,
    const std::vector< Channel >& channels,
    const bool                    isPerceptual
)
{
    ColorSpaceInfo infos;
    infos._isPerceptual = isPerceptual;
    infos._name = name;
    infos._nbChannels = static_cast< unsigned int >( channels.size() );
    infos._firstChannel = channels[0];
    _colorSpaceInfo[colorSpace] = infos;

    for( std::vector< Channel >::const_iterator it( channels.begin() );
         it != channels.end(); ++it ) {
        _isNative[colorSpace][*it] = true;
    }
}

///////////////////////////////////////////////////////////////////////////////

void ColorSpaces::initialize()
{
    std::vector< Channel > channels;
 
    // RGB
    channels.clear();
    channels.push_back( R );
    channels.push_back( G );
    channels.push_back( B );
    registerColorSpace( RGB, "RGB", channels, false );

    // CMY
    channels.clear();
    channels.push_back( Cy );
    channels.push_back( Ma );
    channels.push_back( Ye );
    registerColorSpace( CMY, "CMY", channels, false );
    
    // XYZ
    channels.clear();
    channels.push_back( X );
    channels.push_back( Y );
    channels.push_back( Z );
    registerColorSpace( XYZ, "XYZ", channels, true );

    // Lab
    channels.clear();
    channels.push_back( L );
    channels.push_back( a );
    channels.push_back( b );
    registerColorSpace( Lab, "L*a*b*", channels, true );

    // HVC
    channels.clear();
    channels.push_back( H );
    channels.push_back( V );
    channels.push_back( C );
    registerColorSpace( HVC, "HVC", channels, true );

    // Grayscale
    channels.clear();
    channels.push_back( Gray );
    registerColorSpace( Grayscale, "Grayscale", channels, false );

    // Indexed
    channels.clear();
    channels.push_back( Index );
    registerColorSpace( Indexed, "Indexed", channels, false );

    _initialized = true;
}

///////////////////////////////////////////////////////////////////////////////

void ColorSpaces::finalize()
{
    assert( _initialized );
    _initialized = false;
}
