// 
//  colorSpaces.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: colorSpaces.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _COLORSPACES_H_
#define _COLORSPACES_H_

#include "pixelDepths.h"

#include <vector>
#include <string>

class ImageBase;

class ColorSpaces
{

public:

    enum ColorSpace {
            RGB,
            CMY,
            XYZ,
            Lab,
            HVC,
            Grayscale,
            Indexed,
            UNKNOWN,
            NB_COLOR_SPACES
    };

    enum Channel {
            R, G, B,
            Cy, Ma, Ye,
            X, Y, Z,
            L, a, b,
            H, V, C,
            Gray,
            Index,
            NB_CHANNELS
    };

    static void initialize();
    static void finalize();

    static unsigned int getNbChannels( ColorSpace colorSpace );
    static bool isPerceptual( ColorSpace colorSpace );
    static bool isNative( ColorSpace colorSpace, Channel channel );
    static int getChannelIndex( ColorSpace colorSpace, Channel channel );
    static const std::string& toString( ColorSpace colorSpace );

private:

    struct ColorSpaceInfo
    {
        bool        _isPerceptual;
        std::string _name;
        short int   _nbChannels;
        Channel     _firstChannel;
    };

    static void registerColorSpace(
        ColorSpace                    colorSpace,
        const std::string&            name,
        const std::vector< Channel >& channels,
        bool                          isPerceptual
    );

    static ColorSpaceInfo _colorSpaceInfo[NB_COLOR_SPACES];
    static bool           _isNative[NB_COLOR_SPACES][NB_CHANNELS];

    static bool _initialized;
};

#include "colorSpaces.inline.h"

#endif /* _COLORSPACES_H_ */
