// 
//  correlogram.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: correlogram.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _CORRELOGRAM_H_
#define _CORRELOGRAM_H_

#include "indexedImage.h"
#include "shared_ptr.h"
#include "pixelDepths.h"

class IndexedImage;
class Histogram;

class Correlogram
{

    typedef unsigned long StorageType;

public:

    Correlogram(
        const IndexedImage*     img,
        unsigned short          distance
    );
    ~Correlogram();

    StorageType getOccurence(
        unsigned long ci,
        unsigned long cj
    ) const;

    float getRatio(
        unsigned long ci,
        unsigned long cj
    ) const;

    unsigned long getNbBins() const;
    unsigned short getDistance() const;

    shared_ptr< const PixelIndexer > getPixelIndexer() const;
    
    float getContrast() const;
    float getUniformity() const;
    float getInverseDifferenceMoment() const;
    float getEntropy() const;

private:

    void fill( const IndexedImage* img );

    void processVicinity(
        const IndexedImage*              img,
        const int                        width,
        const int                        height,
        const long                       i,
        const long                       j,
        IndexedImage::StorageType        ci
    );

    StorageType**  _occurences;
    StorageType*   _paths;
    unsigned long  _nbColors;
    unsigned short _distance;

    shared_ptr< const PixelIndexer > _indexer;
    
    void computeContrast();
    void computeUniformity();
    void computeInverseDifferenceMoment();
    void computeEntropy();
    
    float _contrast;
    float _uniformity;
    float _inverseDifferenceMoment;
    float _entropy;

    // Not allowed
    Correlogram( const Correlogram& );
    Correlogram& operator=( const Correlogram& );
};

#include "correlogram.inline.h"

#endif /* _CORRELOGRAM_H_ */
