// 
//  correlogramFeature.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: correlogramFeature.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "correlogramFeature.h"
#include "correlogram.h"
#include "pixelIndexer.h"

///////////////////////////////////////////////////////////////////////////////

int CorrelogramFeature::getNbStatistics()
{
    return 4;
}

///////////////////////////////////////////////////////////////////////////////

ImageFeature::Type CorrelogramFeature::getType() const
{
    return CORRELOGRAM;
}

///////////////////////////////////////////////////////////////////////////////

ColorSpaces::ColorSpace CorrelogramFeature::getColorSpace() const
{
    return _colorSpace;
}

///////////////////////////////////////////////////////////////////////////////

CorrelogramFeature::CorrelogramFeature(
    const IndexedImage* img,
    unsigned char       distances
) : _colorSpace( img->getPixelIndexer()->getTargetColorSpace() )
{
    float contrast      = 0.0f;
    float uniformity    = 0.0f;
    float invDiffMoment = 0.0f;
    float entropy       = 0.0f;
    
    int n = 0;
    for( int i = 0; i < 4; ++i) {
        if( distances & ( 1 << i ) ) {
            // We sum the statistics
            Correlogram corr( img, i + 1 );
            contrast      += corr.getContrast();
            uniformity    += corr.getUniformity();
            invDiffMoment += corr.getInverseDifferenceMoment();
            entropy       += corr.getEntropy();
            ++n;
        }
    }

    assert( n > 0 );

    // We average the statistics for all the distances we have
    addStatistic( contrast / n );
    addStatistic( uniformity / n );
    addStatistic( invDiffMoment / n );
    addStatistic( entropy / n );
}

///////////////////////////////////////////////////////////////////////////////

CorrelogramFeature::CorrelogramFeature(
    std::istream&           is,
    ColorSpaces::ColorSpace colorSpace
) : TextureFeature( is, getNbStatistics() ),
    _colorSpace( colorSpace )
{}
