// 
//  histogram.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: histogram.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "histogram.h"
#include "indexedImage.h"
#include "image.h"
#include "pixelIndexer.h"

#include <iostream>
#include <iterator>
#include <numeric>
#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

Histogram::Histogram(
    Image< STUInt8 >*                  img,
    shared_ptr< const PixelIndexer > indexer
)
: _bins( indexer->getNbBins(), 0 ),
  _indexer( indexer )
{
    assert( _indexer->getSourceColorSpace() == img->getColorSpace() );

    const Image< STUInt8 >::StorageType* src( img->getPixels() );
    const unsigned long nbPixels( img->getWidth() * img->getHeight() );
    const int pixelSize( img->getNbChannels() );

    unsigned long index;
    for( unsigned long i = 0; i < nbPixels; ++i ) {
        index = _indexer->getIndex( src  );
        ++_bins[ index ];
        src += pixelSize;
    }
}

///////////////////////////////////////////////////////////////////////////////

Histogram::Histogram( IndexedImage* img )
: _bins(),
  _indexer( img->getPixelIndexer() )
{
    const unsigned long nbBins( _indexer->getNbBins() );
    _bins.reserve( nbBins );
    std::fill_n( std::back_inserter( _bins ), nbBins, 0 );

    const IndexedImage::StorageType* src( img->getPixels() );
    const unsigned long nbPixels( img->getWidth() * img->getHeight() );

    for( unsigned long i = 0; i < nbPixels; ++i ) {
      ++_bins[ *(src++) ];
    }      
}

///////////////////////////////////////////////////////////////////////////////

Histogram::~Histogram()
{}

///////////////////////////////////////////////////////////////////////////////

unsigned int Histogram::getNbBins() const
{
    return static_cast< unsigned int >( _bins.size() );
}

///////////////////////////////////////////////////////////////////////////////

unsigned int Histogram::getBinCount( unsigned int bin ) const
{
    return _bins[ bin ];
}

///////////////////////////////////////////////////////////////////////////////

unsigned long Histogram::getTotalCount() const
{
    return std::accumulate( _bins.begin(), _bins.end(), 0 );
}

///////////////////////////////////////////////////////////////////////////////

Histogram::BinIterator Histogram::begin() const
{
    return _bins.begin();
}

///////////////////////////////////////////////////////////////////////////////

Histogram::BinIterator Histogram::end() const
{
    return _bins.end();
}

///////////////////////////////////////////////////////////////////////////////

shared_ptr< const PixelIndexer > Histogram::getPixelIndexer() const
{
    return _indexer;
}
