// 
//  histogram.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: histogram.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _HISTOGRAM_H_
#define _HISTOGRAM_H_

#include "pixelDepths.h"
#include "shared_ptr.h"
#include <vector>

template< PixelDepth D > class Image;
class PixelIndexer;
class IndexedImage;

class Histogram
{

public:

    typedef unsigned int StorageType;
    typedef std::vector< StorageType > BinArray;
    typedef BinArray::const_iterator BinIterator;

    Histogram(
        Image< STUInt8 >*                  img,
        shared_ptr< const PixelIndexer > indexer
    );
    
    explicit Histogram( IndexedImage* img );

    virtual ~Histogram();

    StorageType getNbBins() const;
    StorageType getBinCount( unsigned int bin ) const;
    unsigned long getTotalCount() const;

    BinIterator begin() const;
    BinIterator end() const;

    shared_ptr< const PixelIndexer > getPixelIndexer() const;

private:

    BinArray                         _bins;
    shared_ptr< const PixelIndexer > _indexer;
};

#endif /* _HISTOGRAM_H_ */
