// 
//  histogramFeature.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: histogramFeature.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _HISTOGRAMFEATURE_H_
#define _HISTOGRAMFEATURE_H_

#include "imageFeature.h"
#include "shared_ptr.h"
#include <vector>

class Histogram;
class PixelIndexer;

class HistogramFeature : public ImageFeature
{

public:

    typedef float StorageType;

    HistogramFeature( const shared_ptr< const Histogram >& histo );

    HistogramFeature(
        std::istream&                    is,
        shared_ptr< const PixelIndexer > indexer
    );

    ~HistogramFeature();

    // See ReportFileComponent
    void write( std::ostream& os ) const;

    // See ImageFeature
    Type getType() const;
    ColorSpaces::ColorSpace getColorSpace() const;

    // Various histogram distances
    float euclideanDistance( const HistogramFeature& left ) const;
    float weightedDistance( const HistogramFeature& left ) const;
    float intersectionDistance( const HistogramFeature& left ) const;
    float manhattanDistance( const HistogramFeature& left ) const;

private:

    std::vector< StorageType >       _bins;
    shared_ptr< const PixelIndexer > _indexer;

    // Prohibited
    HistogramFeature( const HistogramFeature& );
    HistogramFeature& operator=( const HistogramFeature& );
};

#endif /* _HISTOGRAMFEATURE_H_ */
