// 
//  histogramRanker.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: histogramRanker.cpp 2928 2008-04-13 22:48:01Z benoit $
// 

#include "histogramRanker.h"
#include "imageReport.h"
#include "histogramFeature.h"

///////////////////////////////////////////////////////////////////////////////

HistogramRanker::HistogramRanker(
    DistanceType type,
    shared_ptr< const HistogramFeature > query
) : _type( type ),
    _query( query )
{}

///////////////////////////////////////////////////////////////////////////////

float HistogramRanker::rank(
    const shared_ptr< const ImageReport >& imgReport
) const
{
    shared_ptr< const HistogramFeature > candidate(
        dynamic_pointer_cast< const HistogramFeature >( imgReport->getFeature(
            ImageFeature::HISTOGRAM, _query->getColorSpace() ) ) );

    switch( _type ) {
        case Euclidean:
            return _query->euclideanDistance( *candidate );
            break;
        case Weighted:
            return _query->weightedDistance( *candidate );
            break;
        case Intersection:
            return _query->intersectionDistance( *candidate );
            break;
        case Manhattan:
            return _query->manhattanDistance( *candidate );
            break;
    }

    return 0.0f;
}
