// 
//  image.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: image.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _IMAGE_H_
#define _IMAGE_H_

#include "colorSpaces.h"
#include "pixelDepths.h"
#include "pixelBuffer.h"

template< PixelDepth D > class Image;

class ImageBase : public ColorSpaces
{

    friend class DirectColorConverter;
    friend class CompositeColorConverter;

public:

    static ImageBase* create(
        unsigned int width,
        unsigned int height,
        ColorSpace   colorSpace,
        PixelDepth   pixelDepth
    );

    unsigned int getWidth() const;
    unsigned int getHeight() const;
    unsigned int getNbChannels() const;
    ColorSpace   getColorSpace() const;
    
    virtual PixelDepth getPixelDepth() const = 0;

    virtual ~ImageBase();

protected:

    ImageBase(
        unsigned int width,
        unsigned int height,
        ColorSpace   colorSpace,
        unsigned int valueSize
    );

    PixelBuffer& getPixelBuffer();
    const PixelBuffer& getPixelBuffer() const;
    unsigned int getLineStride() const;
    unsigned char* getPixels( int x = 0, int y = 0);
    const unsigned char* getPixels( int x = 0, int y = 0) const;

private:

    unsigned int _width;
    unsigned int _height;
    ColorSpace   _colorSpace;
    PixelBuffer  _pixelBuffer;

};

template< PixelDepth D  >
class Image : public ImageBase
{

    friend class ImageBase;

public:

    typename depth_traits< D >::storage_type typedef StorageType;

    StorageType getValue( int x, int y, Channel channel ) const;
    void setValue( StorageType value, int x, int y, Channel channel );
    
    StorageType* getPixels( int x = 0, int y = 0 );
    const StorageType* getPixels( int x = 0, int y = 0 ) const;

    PixelDepth getPixelDepth() const;
    unsigned int getLineStride() const;

    ~Image();

private:

    Image(
        unsigned int width,
        unsigned int height,
        ColorSpace   colorSpace
    );
};

#include "image.inline.h"

#endif /* _IMAGE_H_ */
