// 
//  image.inline.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: image.inline.h 2926 2008-04-13 22:45:08Z benoit $
// 

#include "colorUtil.h"
#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

inline
unsigned int ImageBase::getWidth() const
{
    return _width;
}

///////////////////////////////////////////////////////////////////////////////

inline
unsigned int ImageBase::getHeight() const
{
    return _height;
}

///////////////////////////////////////////////////////////////////////////////

inline
ColorSpaces::ColorSpace ImageBase::getColorSpace() const
{
    return _colorSpace;
}

///////////////////////////////////////////////////////////////////////////////

inline
unsigned int ImageBase::getNbChannels() const
{
    return ColorSpaces::getNbChannels( _colorSpace );
}

///////////////////////////////////////////////////////////////////////////////

inline
unsigned int ImageBase::getLineStride() const
{
    return _pixelBuffer.getLineStride();
}

///////////////////////////////////////////////////////////////////////////////

inline
const unsigned char* ImageBase::getPixels(
    int x /* = 0 */,
    int y /* = 0 */
) const
{
    return _pixelBuffer.getPixels( x, y );
}

///////////////////////////////////////////////////////////////////////////////

inline
unsigned char* ImageBase::getPixels(
    int x /* = 0 */,
    int y /* = 0 */
)
{
    return _pixelBuffer.getPixels( x, y );
}

///////////////////////////////////////////////////////////////////////////////

inline
PixelBuffer& ImageBase::getPixelBuffer()
{
    return _pixelBuffer;
}

///////////////////////////////////////////////////////////////////////////////

inline
const PixelBuffer& ImageBase::getPixelBuffer() const
{
    return _pixelBuffer;
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Image< D >::Image(
    unsigned int width,
    unsigned int height,
    ColorSpace   colorSpace
) : ImageBase( width, height,
               colorSpace,
               sizeof( typename Image< D >::StorageType ) )
{}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
Image< D >::~Image()
{}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
inline
PixelDepth Image< D >::getPixelDepth() const
{
    return D;
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
inline
unsigned int Image< D >::getLineStride() const
{
    // Be safe for StorageType pointer arithmetic
    return ImageBase::getLineStride() /
           sizeof( typename Image< D >::StorageType );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
inline
typename Image< D >::StorageType*
Image< D >::getPixels(
    int x /* = 0 */,
    int y /* = 0 */
)
{
    return reinterpret_cast< typename Image< D >::StorageType* >(
        ImageBase::getPixels( x, y ) );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
inline
const typename Image< D >::StorageType*
Image< D >::getPixels(
    int x /* = 0 */,
    int y /* = 0 */
) const
{
    return reinterpret_cast< const typename Image< D >::StorageType* >(
        ImageBase::getPixels( x, y ) );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
inline
typename Image< D >::StorageType
Image< D >::getValue( int x, int y, ColorSpaces::Channel c ) const
{
    assert( ColorSpaces::isNative( getColorSpace(), c ) );
    
    return *( reinterpret_cast< const typename Image< D >::StorageType* >(
                ImageBase::getPixels( x, y )  )
                + ColorSpaces::getChannelIndex( getColorSpace(),c )
            );
}

///////////////////////////////////////////////////////////////////////////////

template< PixelDepth D >
inline
void
Image< D >::setValue( StorageType value, int x, int y, ColorSpaces::Channel c )
{
    assert( ColorSpaces::isNative( getColorSpace(), c ) );

    *( reinterpret_cast< typename Image< D >::StorageType* >(
        ImageBase::getPixels( x, y )  )
        + ColorSpaces::getChannelIndex( getColorSpace(),c ) )
        = value;
}

///////////////////////////////////////////////////////////////////////////////

template<>
inline
Image< STUInt8 >::StorageType*
Image< STUInt8 >::getPixels(
    int x /* = 0 */,
    int y /* = 0 */
)
{
    return ImageBase::getPixels( x, y );
}

///////////////////////////////////////////////////////////////////////////////

template<>
inline
Image< STUInt8 >::StorageType
Image< STUInt8 >::getValue( int x, int y, ColorSpaces::Channel c ) const
{
    // No need to reinterpret_cast
    return *( ImageBase::getPixels( x, y )
              + ColorSpaces::getChannelIndex( getColorSpace(),c ) );
}
