// 
//  imageReport.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: imageReport.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "imageReport.h"
#include "imageFeature.h"
#include "stringUtils.h"
#include <iostream>
#include <fstream>
#include <algorithm>

///////////////////////////////////////////////////////////////////////////////

struct CompareType
{
    CompareType( ImageFeature::Type type )
    : _type( type )
    {};

    bool operator()( const ImageReport::FeaturePtr& feature )
    {
        return feature->getType() == _type;
    }

    ImageFeature::Type _type;
};

///////////////////////////////////////////////////////////////////////////////

struct CompareTypeAndColorSpace
{
    CompareTypeAndColorSpace(
        ImageFeature::Type type,
        ColorSpaces::ColorSpace colorspace
    ) : _type( type ), _colorSpace( colorspace )
    {};

    bool operator()( ImageReport::FeaturePtr feature )
    {
        return feature->getType() == _type &&
               feature->getColorSpace() == _colorSpace;
    }

    ImageFeature::Type      _type;
    ColorSpaces::ColorSpace _colorSpace;
};

///////////////////////////////////////////////////////////////////////////////

ImageReport::ImageReport( const std::wstring& filename, int nbFeatures )
: _filename( filename ),
  _features()
{
    _features.reserve( nbFeatures );
}

///////////////////////////////////////////////////////////////////////////////

void ImageReport::add( FeaturePtr feature  )
{
    _features.push_back( feature );
}

///////////////////////////////////////////////////////////////////////////////

ImageReport::~ImageReport()
{}

///////////////////////////////////////////////////////////////////////////////

const std::wstring& ImageReport::getFilename() const
{
    return _filename;
}

///////////////////////////////////////////////////////////////////////////////

void ImageReport::write( std::ostream& os ) const
{
    // Write filename
    os << wstr2str( _filename ) << '\0';

    // Write features
    for( std::vector< FeaturePtr >::const_iterator it( _features.begin() );
         it != _features.end(); ++it ) {
        (*it)->write( os );
    }
}

///////////////////////////////////////////////////////////////////////////////

ImageReport::FeaturePtr ImageReport::getFeature(
    ImageFeature::Type      type,
    ColorSpaces::ColorSpace colorSpace
)
{
    std::vector< FeaturePtr >::const_iterator it;
    if( colorSpace == ColorSpaces::UNKNOWN ) {
        it = std::find_if( _features.begin(), _features.end(),
                           CompareType( type ) );
    }
    else {
        it = std::find_if( _features.begin(),
                           _features.end(),
                           CompareTypeAndColorSpace( type, colorSpace ) );
    }

    return ( it == _features.end() ? FeaturePtr() : *it );
}

///////////////////////////////////////////////////////////////////////////////

ImageReport::FeaturePtrConst ImageReport::getFeature(
    ImageFeature::Type      type,
    ColorSpaces::ColorSpace colorSpace
) const
{
    std::vector< FeaturePtr >::const_iterator it;
    if( colorSpace == ColorSpaces::UNKNOWN ) {
        it = std::find_if( _features.begin(), _features.end(),
                           CompareType( type ) );
    }
    else {
        it = std::find_if( _features.begin(),
                           _features.end(),
                           CompareTypeAndColorSpace( type, colorSpace ) );
    }

    return ( it == _features.end() ? FeaturePtr() : *it );
}
