// 
//  imageReport.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: imageReport.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _IMAGEREPORT_H_
#define _IMAGEREPORT_H_

#include "reportFileComponent.h"
#include "imageFeature.h"
#include "colorSpaces.h"
#include "shared_ptr.h"
#include <istream>
#include <vector>
#include <string>

class ImageReport : public ReportFileComponent
{

public:

    typedef shared_ptr< ImageFeature > FeaturePtr;
    typedef shared_ptr< const ImageFeature > FeaturePtrConst;

    ~ImageReport();

    const std::wstring& getFilename() const;

    // See ReportFileComponent
    void write( std::ostream& os ) const;

    // Retrieve a feature by its type
    FeaturePtr getFeature(
        ImageFeature::Type      type,
        ColorSpaces::ColorSpace colorSpace = ColorSpaces::UNKNOWN
    );
    FeaturePtrConst getFeature(
        ImageFeature::Type      type,
        ColorSpaces::ColorSpace colorSpace = ColorSpaces::UNKNOWN
    ) const;

private:

    friend class ImageReportFactory;

    // See ImageReportFactory
    ImageReport( const std::wstring& filename, int nbFeatures );
    void add( FeaturePtr feature );

    std::wstring              _filename;
    std::vector< FeaturePtr > _features;
};

#endif /* _IMAGEREPORT_H_ */
