// 
//  kMeansPixelIndexer.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: kMeansPixelIndexer.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _KMEANSPIXELINDEXER_H_
#define _KMEANSPIXELINDEXER_H_

#include "pixelIndexer.h"
#include "staticPixelIndexer.h"
#include "colorSpaces.h"
#include "pixelDepths.h"
#include "scoped_ptr.h"

class KMeansPixelIndexer : public PixelIndexer
{
    
public:

    static KMeansPixelIndexer* fromParams( int64_t params );
    static KMeansPixelIndexer* fromParams( std::istream& is, int64_t params );

    KMeansPixelIndexer(
        unsigned short splitR,
        unsigned short splitG,
        unsigned short splitB,
        unsigned short nbClass
    );

    KMeansPixelIndexer(
        std::istream&  is,
        unsigned short splitR,
        unsigned short splitG,
        unsigned short splitB,
        unsigned short nbClass
    );

    ~KMeansPixelIndexer();
    
    // See PixelIndexer
    ColorSpaces::ColorSpace getSourceColorSpace() const;
    ColorSpaces::ColorSpace getTargetColorSpace() const;
    PixelIndexer::BinIndexStorageType getIndex( const PixelStorageType* pixel ) const;
    const Color< STFloat32 >& getBinCenter( unsigned int bin ) const;

    // Write the bin LUT and bin centers to the stream
    void write( std::ostream& is ) const;

private:

    void computeKMeans();

    const unsigned short _splitR;
    const unsigned short _splitG;
    const unsigned short _splitB;
    const unsigned short _nbClass;

    std::vector< int >   _vecClasses;

    std::vector< Color< STFloat32 > > _binCenters;

    scoped_ptr< StaticPixelIndexer > _staticIndexer;
};


#endif /* _KMEANSPIXELINDEXER_H_ */
