// 
//  OSUtils.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: osUtils.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "osUtils.h"

#ifdef CPU_POWERPC
    #include <sys/types.h>
    #include <sys/sysctl.h>
#endif

#include <wx/thread.h>

#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

OSUtils::Endianness detectEndianness()
{
   short int word = 0x0001;
   char *byte = (char *) &word;
   return( byte[0] ? OSUtils::LittleEndian : OSUtils::BigEndian );
};

///////////////////////////////////////////////////////////////////////////////

bool detectAltiVec()
{
#ifdef __VEC__
    int result = 0;
    size_t size = sizeof( result );

    // Altivec is an optional feature on PowerPC CPUs
    return ( sysctlbyname( "hw.optional.altivec",
                           &result,
                           &size,
                           NULL,
                           NULL) == 0 );
#else
    return false;
#endif
}

///////////////////////////////////////////////////////////////////////////////

OSUtils::Endianness OSUtils::_endianness = Unknown;
bool OSUtils::_hasAltivec = false;
bool OSUtils::_initialized = false;

///////////////////////////////////////////////////////////////////////////////

void OSUtils::initialize()
{
    _endianness = detectEndianness();
    _hasAltivec = detectAltiVec();
    _initialized = true;
}

///////////////////////////////////////////////////////////////////////////////

void OSUtils::finalize()
{
    _initialized = false;
}

///////////////////////////////////////////////////////////////////////////////

OSUtils::Endianness OSUtils::getEndianness()
{
    assert( _initialized );
    return _endianness;
}

///////////////////////////////////////////////////////////////////////////////

bool OSUtils::isLittleEndian()
{
    assert( _initialized );
    return _endianness == LittleEndian;
}

///////////////////////////////////////////////////////////////////////////////

bool OSUtils::isBigEndian()
{
    assert( _initialized );
    return _endianness == BigEndian;
}

///////////////////////////////////////////////////////////////////////////////

bool OSUtils::hasAltivec()
{
   assert( _initialized );
   return _hasAltivec;
}

///////////////////////////////////////////////////////////////////////////////

int OSUtils::getCPUcount()
{
    return wxThread::GetCPUCount();
}
