// 
//  pixelIndexer.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: pixelIndexer.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "pixelIndexer.h"
#include "color.h"

///////////////////////////////////////////////////////////////////////////////

PixelIndexer::PixelIndexer( BinIndexStorageType nbBins )
: _distanceMatrixReady( false ),
  _distanceMatrix( 0 ),
  _nbBins( nbBins )
{}

///////////////////////////////////////////////////////////////////////////////

PixelIndexer::~PixelIndexer()
{
    if( _distanceMatrixReady ) {
        // Free memory used by distance matrix
        for( BinIndexStorageType i = 0; i < _nbBins; ++i ) {
            delete[] _distanceMatrix[i];
        }
        delete[] _distanceMatrix;
    }
}

///////////////////////////////////////////////////////////////////////////////

PixelIndexer::BinIndexStorageType PixelIndexer::getNbBins() const
{
    return _nbBins;
}

///////////////////////////////////////////////////////////////////////////////

float PixelIndexer::getDist(
    PixelIndexer::BinIndexStorageType b1,
    PixelIndexer::BinIndexStorageType b2
) const
{
    assert ( b1 < getNbBins() && b2 < getNbBins() );
    assert( _distanceMatrixReady );

    return _distanceMatrix[b1][b2];
}

///////////////////////////////////////////////////////////////////////////////

void PixelIndexer::fillDistancesMatrix()
{
    assert( !_distanceMatrixReady );

    // Note: the distances matrix is symetric
    _distanceMatrix = new float*[_nbBins];
    float distanceMax( 0.0f );
    for( BinIndexStorageType i = 0; i < _nbBins; ++i ) {
        _distanceMatrix[i] = new float[ _nbBins];
        for( BinIndexStorageType j = 0; j < _nbBins; ++j ) {
            Color< STFloat32 > ci( getBinCenter( i ) );
            Color< STFloat32 > cj( getBinCenter( j ) );
            _distanceMatrix[i][j] = ( ci - cj ).norm();
            
            if (_distanceMatrix[i][j] > distanceMax)
                distanceMax = _distanceMatrix[i][j];
        }
    }
    
    // Normalize the distance matrix
    for( BinIndexStorageType i = 0; i < _nbBins; ++i ) {
        for( BinIndexStorageType j = 0; j < ( _nbBins - i ); ++j ) {
            _distanceMatrix[i][j] = _distanceMatrix[i][j] / distanceMax;
        }
    }

    _distanceMatrixReady = true;
}
