// 
//  pixelIndexer.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: pixelIndexer.h 2926 2008-04-13 22:45:08Z benoit $
// 

#ifndef _PIXELINDEXER_H_
#define _PIXELINDEXER_H_

#include "colorSpaces.h"
#include "pixelDepths.h"

template< PixelDepth D > class Color;

class PixelIndexer
{

protected:

   typedef depth_traits< STUInt8 >::storage_type PixelStorageType; 
   typedef unsigned int BinIndexStorageType;

public:

    virtual ~PixelIndexer();

    BinIndexStorageType getNbBins() const;

    virtual ColorSpaces::ColorSpace getSourceColorSpace() const = 0;
    virtual ColorSpaces::ColorSpace getTargetColorSpace() const = 0;

    virtual unsigned int getIndex( const PixelStorageType* pixel ) const = 0;
    virtual const Color< STFloat32 >&
    getBinCenter( BinIndexStorageType bin ) const = 0;

    float getDist( BinIndexStorageType b1, BinIndexStorageType b2) const;

protected:

    PixelIndexer( BinIndexStorageType nbBins );
    void fillDistancesMatrix();

private:

    bool                      _distanceMatrixReady;
    float**                   _distanceMatrix;
    const BinIndexStorageType _nbBins;

    // Prohibited
    PixelIndexer( const PixelIndexer& );
    PixelIndexer& operator=( const PixelIndexer& );

};

#endif /* _PIXELINDEXER_H_ */
