// 
//  ppmImageFile.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: ppmImageFile.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "ppmImageFile.h"
#include "image.h"
#include "stringUtils.h"
#include <fstream>

///////////////////////////////////////////////////////////////////////////////

void skipWSandComments( std::istream& input )
{
    char c;
    do {
        c = input.peek();
        if( c == '#' ) {
            while( input.peek() != '\n' ) {
                input.get();
            }
            input.get();
        }
        else if( c == ' ' || c == '\n' || c == '\t' || c == '\r' ) {
            input.get();
        }
        else {
            return;
        }
    }
    while( !input.eof() );
    throw( "Unexpected EOF");
}

///////////////////////////////////////////////////////////////////////////////

void readBinaryData( std::istream& input, Image< STUInt8 >* image )
{
    const int width = image->getWidth();
    const int height = image->getHeight();
    const int nbChannels = image->getNbChannels();

    unsigned long dataSize( width * height * nbChannels  );

    // Make sure we have enough data
    unsigned long curPos = input.tellg();
    input.seekg( 0, std::ios::end );
    unsigned long eofPos = input.tellg();
    input.seekg( curPos, std::ios::beg );
    if( ( eofPos - curPos ) < dataSize ) {
        throw( "Image data is too short" );
    }
  
    // Read all pixels
    input.read( reinterpret_cast< char* >( image->getPixels() ), dataSize );
}

///////////////////////////////////////////////////////////////////////////////

PPMImageFile::PPMImageFile( const std::wstring& filePath )
 : ImageFile( filePath ),
   _storageType( Unknown )
{}

///////////////////////////////////////////////////////////////////////////////

PPMImageFile::~PPMImageFile()
{}

///////////////////////////////////////////////////////////////////////////////

ImageBase* PPMImageFile::read() const
{
    // Open the input file stream
    std::ifstream input;
    input.open(
        wstr2str( getFilePath() ).c_str(),
        std::ifstream::in | std::ifstream::binary
    );

    if( input.bad() ) {
        throw "Could not open file for writing";
    }

    // Format variables
    unsigned int width;
    unsigned int height;

    // Magic number
    char magicNumber[2];
    input.get( magicNumber, 2+1 );

    if( magicNumber[0] != 'P') {
        throw "Invalid magic number";
    }

    switch( magicNumber[1] )
    {
        case '6':
            _storageType = Binary;
            break;
        case '3':
            _storageType = ASCII;
            break;
    }

    if( _storageType == Unknown ) {
        throw "Invalid magic number";
    }

    // Width
    skipWSandComments( input );
    input >> width;

    // Height
    skipWSandComments( input );
    input >> height;

    // Maxval
    skipWSandComments( input );
    int maxVal;
    input >> maxVal;
    if( maxVal > 255 ) {
        throw( "Images with more than 8 bit are not supported");
    }

    // Whitespace
    input.get();

    // Output image
    Image< STUInt8 >* image( dynamic_cast< Image< STUInt8 >* >( 
        ImageBase::create( width, height, ColorSpaces::RGB, STUInt8 ) ) );

    // Read image data
    if( _storageType == Binary ) {
        readBinaryData( input, image );
    }
    else {
        throw( "Can only read binary PPM files");
    }

    return image;
}

///////////////////////////////////////////////////////////////////////////////

void PPMImageFile::write( const ImageBase* baseImage )
{
    const Image< STUInt8 >* img(
        dynamic_cast< const Image< STUInt8 >* >( baseImage ) );
    assert( img != 0 );
    assert( img->getColorSpace() == ColorSpaces::RGB );

    // Open file for writing
    std::ofstream file;
	file.open( wstr2str( getFilePath() ).c_str(),
		std::ifstream::out | std::ifstream::binary );

    // Magic number
    file << "P6" << std::endl;

    // Width
    file << img->getWidth();
    file << " ";

    // Height
    file << img->getHeight();
    file << std::endl;

    // Comment
    file << "#" << wstr2str( getFilePath() ) << std::endl;

    // Maxval
    file << 255 << '\n';

    // Write data
    file.write(
        reinterpret_cast< const char* >( img->getPixels() ),
        img->getWidth() * img->getHeight() * img->getNbChannels() );
}
