// 
//  staticPixelIndexer.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: staticPixelIndexer.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "staticPixelIndexer.h"
#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

StaticPixelIndexer* StaticPixelIndexer::fromParams( int64_t params )
{
    //  val: |r|g|b|n   |d|
    // byte: |0|1|2|3456|7|
    unsigned char* params_char;
    params_char = reinterpret_cast< unsigned char* >( &params );

    unsigned char splitR = params_char[0];
    unsigned char splitG = params_char[1];
    unsigned char splitB = params_char[2];

    return new StaticPixelIndexer( splitR, splitG, splitB );
}

///////////////////////////////////////////////////////////////////////////////

StaticPixelIndexer::StaticPixelIndexer(
    unsigned short splitR,
    unsigned short splitG,
    unsigned short splitB
) : PixelIndexer( splitR * splitG * splitB ),
    _splitR( splitR ),
    _splitG( splitG ),
    _splitB( splitB )
{
    assert( splitR > 0 );
    assert( splitG > 0 );
    assert( splitB > 0 );

    // Calculate all bin centers
    depth_traits< STFloat32 >::storage_type center[3];
    _binCenters.reserve( getNbBins() );
    const ColorSpaces::ColorSpace colorSpace( getTargetColorSpace() );
    for( unsigned short i = 0; i < _splitR; ++i ) {
        for( unsigned short j = 0; j < _splitG; ++j ) {
            for( unsigned short k = 0; k < _splitB; ++k ) {
                center[0] = ( 1.0f / _splitR ) * ( i + 0.5f );
                center[1] = ( 1.0f / _splitG ) * ( j + 0.5f );
                center[2] = ( 1.0f / _splitB ) * ( k + 0.5f );
                _binCenters.push_back(
                    Color< STFloat32 >( colorSpace, center ) );
            }
        }
    }

    // Compute cross distances
    fillDistancesMatrix();
}
