// 
//  textureFeature.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: textureFeature.cpp 2928 2008-04-13 22:48:01Z benoit $
// 

#include "textureFeature.h"
#include <math.h>
#include <assert.h>

///////////////////////////////////////////////////////////////////////////////

void TextureFeature::write( std::ostream& os ) const
{
    // Dump the statistics vector
    os.write( reinterpret_cast< const char* >( &( _statistics[0] ) ),
              sizeof( float ) * _statistics.size() );
}

///////////////////////////////////////////////////////////////////////////////

TextureFeature::TextureFeature()
{}

///////////////////////////////////////////////////////////////////////////////

TextureFeature::TextureFeature( std::istream& is, size_t nbStats )
: _statistics( nbStats, 0.0f )
{
    // Read nbStats statistic values
    is.read( reinterpret_cast< char* >( &( _statistics[0] ) ),
             sizeof( float ) * nbStats );
}

///////////////////////////////////////////////////////////////////////////////

TextureFeature::~TextureFeature()
{}

///////////////////////////////////////////////////////////////////////////////

void TextureFeature::addStatistic( float stat )
{
    _statistics.push_back( stat );
}

///////////////////////////////////////////////////////////////////////////////

float TextureFeature::manhattanDistance( const TextureFeature& left ) const
{
    assert( left._statistics.size() == _statistics.size() );

    float dist = 0;
    for (unsigned int i = 0; i < _statistics.size(); ++i)
    {
        dist += fabs(_statistics[i] - left._statistics[i]);
    }
    
    return dist;
}

///////////////////////////////////////////////////////////////////////////////

float TextureFeature::euclideanDistance( const TextureFeature& left ) const
{
    assert( left._statistics.size() == _statistics.size() );

    float dist = 0;
    for (unsigned int i = 0; i < _statistics.size(); ++i)
    {
        dist += (_statistics[i] - left._statistics[i]) 
                    * (_statistics[i] - left._statistics[i]);
    }
    
    return sqrtf(dist);
}

///////////////////////////////////////////////////////////////////////////////

void TextureFeature::normalize( const MinMaxVector& minMaxValues )
{
    assert( _statistics.size() == minMaxValues.size() );

    for (unsigned int i = 0; i < _statistics.size(); ++i) {
        _statistics[i]  = _statistics[i] - minMaxValues[i].first;
        _statistics[i] /= minMaxValues[i].second - minMaxValues[i].first;
    }
}

///////////////////////////////////////////////////////////////////////////////

float TextureFeature::getStatValue( unsigned int i ) const
{
    assert ( i < _statistics.size() );
    
    return _statistics[i];
}
