// 
//  textureFeatureRangeFinder.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: textureFeatureRangeFinder.cpp 2928 2008-04-13 22:48:01Z benoit $
// 

#include "textureFeatureRangeFinder.h"
#include "coOccurenceMatrixFeature.h"
#include "correlogramFeature.h"
#include "imageReport.h"

///////////////////////////////////////////////////////////////////////////////

TextureFeatureRangeFinder::TextureFeatureRangeFinder( ImageFeature::Type type )
: _type( type ),
  _minMaxPairs()
{
    assert(type == ImageFeature::CO_OCCURENCE
            || type == ImageFeature::CORRELOGRAM);
    
    switch(type)
    {
        case ImageFeature::CO_OCCURENCE:
            _minMaxPairs.reserve(CoOccurenceMatrixFeature::getNbStatistics());
            for (int i = 0;
                 i < CoOccurenceMatrixFeature::getNbStatistics();
                 ++i)
            {
                _minMaxPairs.push_back(
                    std::pair < float, float > (999999999.0, -999999999.0) );
            }
            break;
        
        case ImageFeature::CORRELOGRAM:
            _minMaxPairs.reserve(CorrelogramFeature::getNbStatistics());
            for (int i = 0;
                 i < CorrelogramFeature::getNbStatistics();
                 ++i)
            {
                _minMaxPairs.push_back(
                    std::pair < float, float > (999999999.0, -999999999.0) );
            }
            break;
    }
}

///////////////////////////////////////////////////////////////////////////////

void TextureFeatureRangeFinder::process( ImageReport& report )
{ 
    const shared_ptr< const TextureFeature > texture(
        dynamic_pointer_cast< const TextureFeature >(
            report.getFeature( _type ) ) );
    
    for (unsigned int i = 0; i < _minMaxPairs.size(); ++i)
    {
        float statValue = texture->getStatValue(i);
        
        if (statValue < _minMaxPairs[i].first)
            _minMaxPairs[i].first = statValue;
        if (statValue > _minMaxPairs[i].second)
            _minMaxPairs[i].second = statValue;
    }
}

///////////////////////////////////////////////////////////////////////////////

TextureFeature::MinMaxVector TextureFeatureRangeFinder::getMinMaxPairs()
{
    return _minMaxPairs;
}
