// 
//  textureRanker.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: textureRanker.cpp 2928 2008-04-13 22:48:01Z benoit $
// 

#include "textureRanker.h"
#include "imageReport.h"
#include "textureFeature.h"

///////////////////////////////////////////////////////////////////////////////

TextureRanker::TextureRanker(
    DistanceType type,
    shared_ptr< const TextureFeature > query
) : _type( type ),
    _query( query )
{}

///////////////////////////////////////////////////////////////////////////////

float TextureRanker::rank(
    const shared_ptr< const ImageReport >& imgReport
) const
{
    shared_ptr< const TextureFeature > candidate(
        dynamic_pointer_cast< const TextureFeature >( imgReport->getFeature(
            _query->getType(), _query->getColorSpace() ) ) );

    switch( _type ) {
        case Euclidean:
            return _query->euclideanDistance( *candidate );
            break;
        case Manhattan:
            return _query->manhattanDistance( *candidate );
            break;
        default:
            throw "Unsupported distance type";
            break;
    }

    return 0.0f;
}
