// 
//  uiColorTab.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiColorTab.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "uiColorTab.h"
#include "uiSplitRGB.h"
#include "uiSplitHVC.h"
#include <wx/spinctrl.h>

///////////////////////////////////////////////////////////////////////////////

enum {
    ID_RGB_CHCKBOX,
    ID_HVC_CHCKBOX,
};

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( UiColorTab, wxPanel )
    EVT_CHECKBOX( ID_RGB_CHCKBOX, UiColorTab::OnEnableRGB )
    EVT_CHECKBOX( ID_HVC_CHCKBOX, UiColorTab::OnEnableHVC )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

UiColorTab::UiColorTab( wxWindow* parent )
: wxPanel( parent ),
  _rgbCheckBox( new wxCheckBox(
                        this,
                        ID_RGB_CHCKBOX,
                        _T("Generate RGB histogram") ) ),
  _hvcCheckBox( new wxCheckBox(
                        this,
                        ID_HVC_CHCKBOX,
                        _T("Generate HVC histogram") ) ),
  _uiSplitRGB( new UiSplitRGB( this ) ),
  _uiSplitHVC( new UiSplitHVC( this ) )
{
  // Vertical layout
  wxBoxSizer* topSizer = new wxBoxSizer( wxVERTICAL );

  const int spacing( 10 );

  // RGB histogram options
  topSizer->Add( _rgbCheckBox, 0, wxALL, spacing );
  topSizer->Add( _uiSplitRGB, 0, wxLEFT, 3*spacing );
  _uiSplitRGB->Disable();

  // HVC histogram options
  topSizer->Add( _hvcCheckBox, 0, wxALL, spacing );
  topSizer->Add( _uiSplitHVC, 0, wxLEFT, 3*spacing );
  _uiSplitHVC->Disable();

  // Set layout
  SetSizer( topSizer );
}

///////////////////////////////////////////////////////////////////////////////

void UiColorTab::OnEnableRGB( wxCommandEvent& event )
{
    _uiSplitRGB->Enable( _rgbCheckBox->GetValue() );
}

///////////////////////////////////////////////////////////////////////////////

void UiColorTab::OnEnableHVC( wxCommandEvent& event )
{
    _uiSplitHVC->Enable( _hvcCheckBox->GetValue() );
}

///////////////////////////////////////////////////////////////////////////////

UiColorTab::Values UiColorTab::getValues() const
{
    Values vals;

    vals._generateRGB = _rgbCheckBox->GetValue();
    vals._generateHVC = _hvcCheckBox->GetValue();
    vals._rgbValues = _uiSplitRGB->getValues();
    vals._hvcValues = _uiSplitHVC->getValues();

    return vals;
}
