#include "uiDigestProgress.h"
#include "uiSearchFrame.h"
#include "digestQueue.h"
#include "imageReportFactory.h"
#include <string>
#include <wx/progdlg.h>

///////////////////////////////////////////////////////////////////////////////

enum
{
    ID_TIMER,
    ID_GAUGE,
    ID_BUTTON_CANCEL,
    ID_TEXT_STATUS
};

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( UiDigestProgress, wxFrame )
    EVT_TIMER( ID_TIMER, UiDigestProgress::OnTimer )
    EVT_BUTTON( ID_BUTTON_CANCEL, UiDigestProgress::OnCancel )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

UiDigestProgress::UiDigestProgress(
    const FSUtils::FileList&                            fileList,
    const shared_ptr< const CollectionReport::Header >& header
)
: wxFrame(
       NULL,
       wxID_ANY,
       _T("Progress"),
       wxDefaultPosition,
       wxSize( 400, 100 ),
       wxCAPTION | wxMINIMIZE_BOX ),
  _queue( new DigestQueue( fileList, header ) ),
  _timer( this, ID_TIMER ),
  _header( header )
{
    const int spacing = 10;
    
    // Vertical sizer
    wxBoxSizer* verticalSizer = new wxBoxSizer( wxVERTICAL );
    
    // Gauge
    _gauge = new wxGauge( this, ID_GAUGE, fileList.size() );
    _gauge->SetValue( 0 );
    verticalSizer->Add( _gauge, 1, wxGROW | wxLEFT | wxRIGHT | wxTOP, spacing );
    
    // Horizontal sizer
    wxBoxSizer* horizontalSizer = new wxBoxSizer( wxHORIZONTAL );
    
    // Satus text
    _statusTxt = new wxStaticText( this, ID_TEXT_STATUS, _T( "initializing..."),
        wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT );
    horizontalSizer->Add( _statusTxt, 1, wxGROW , spacing );
    
    // Cancel button
    _cancelBtn = new wxButton( this, ID_BUTTON_CANCEL, _T( "Cancel" ) );
    horizontalSizer->Add( _cancelBtn );
    
    verticalSizer->Add( horizontalSizer, 0,
        wxGROW | wxLEFT | wxRIGHT | wxTOP | wxBOTTOM, spacing );
    
    SetSizer( verticalSizer );
    
    // Start the timer
    _timer.Start( 250 );
}

///////////////////////////////////////////////////////////////////////////////

void UiDigestProgress::OnTimer( wxTimerEvent &event )
{
    const int n = _gauge->GetRange();
    const int i = n - _queue->getSize();
    _gauge->SetValue( i );
    _statusTxt->SetLabel( wxString::Format( _T( "indexing %d/%d..." ), i, n ) );
    
    if( i == n ) {
        _gauge->Pulse();
        OnComplete();
    }
}

///////////////////////////////////////////////////////////////////////////////

void UiDigestProgress::OnCancel( wxCommandEvent &event )
{
    _queue->abort();
    Close( true );
}

///////////////////////////////////////////////////////////////////////////////

void UiDigestProgress::OnComplete()
{
    _timer.Stop();
    
    // Get the collection
    shared_ptr< CollectionReport > report( _queue->getCollectionReport() );
    
    #if 0
    // Save the collection
    wxString defaultPath = _T( "/" );
    wxString defaultFilename = _T( "myDatabase.idb" );
    wxString wildcard = _T( "IDB files (*.idb)|*.idb" );

    wxFileDialog dialog(
        this,
        _T( "Save database as" ),
        defaultPath,
        defaultFilename,
        wildcard,
        wxSAVE | wxOVERWRITE_PROMPT
    );

    wxString reportSavePath;
    if ( dialog.ShowModal() == wxID_OK ) {
        reportSavePath = dialog.GetPath();
    }
    else {
        Close( true );
        return;
    }
    
    // Open file for writing
    std::ofstream file;
    file.open(
        reportSavePath.fn_str(),
        std::ifstream::out | std::ifstream::binary
    );
    assert( file.good() );
    
    // Write the file header
    imgReportFactory->writeHeader( file );
    
    // Write the image reports
    report->write( file );
    #endif
    
    // Load the collection
    UiSearchFrame *searchFrame = new UiSearchFrame( 0 );
    searchFrame->SetCollection( _header, report );
    searchFrame->Show( true );
    
    Close( true );
}
