// 
//  uiGridItem.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiGridItem.cpp 2928 2008-04-13 22:48:01Z benoit $
// 

#include "uiGridItem.h"
#include "searchResult.h"
#include <wx/image.h>

///////////////////////////////////////////////////////////////////////////////

IMPLEMENT_CLASS( UiGridItem, wxThumbnailItem )

///////////////////////////////////////////////////////////////////////////////

UiGridItem::UiGridItem( const SearchResult& result, float scale )
: wxThumbnailItem( result.getFilename().c_str() )
{
    Init();
    Create( result, scale );
}

///////////////////////////////////////////////////////////////////////////////

UiGridItem::UiGridItem(const UiGridItem& item)
: wxThumbnailItem( item.getName() )
{
    Init();
    Create( item._result, item._scale );
}

///////////////////////////////////////////////////////////////////////////////

void UiGridItem::operator=( const UiGridItem& item )
{
    Init();
    Create( item._result, item._scale );
}

///////////////////////////////////////////////////////////////////////////////

UiGridItem::UiGridItem()
{
    Init();
}

///////////////////////////////////////////////////////////////////////////////


UiGridItem::~UiGridItem()
{
    if ( _image != 0 )
        delete _image;
    if ( _thumbnail != 0 ) delete _thumbnail;
}

///////////////////////////////////////////////////////////////////////////////

void UiGridItem::Init()
{
    _result = SearchResult();
    _image = 0;
    _thumbnail = 0;
    _scale = 1.0f;
}

///////////////////////////////////////////////////////////////////////////////

bool UiGridItem::Create( const SearchResult& result, float scale )
{
    _result = result;
    _scale = scale;

    if ( !wxFileExists( getName() ) ) {
        return false;
    }
        
    // load image -> image
    if( _image != 0 ) {
        delete _image;
    }

    _image = new wxImage( getName() );
    // scale image -> thumbnail
    if ( _thumbnail != 0) {
        delete _thumbnail;
    }
    _thumbnail = new wxBitmap(
        _image->Scale( (int) ( _image->GetWidth() * _scale ),
                       (int) ( _image->GetHeight() * _scale ) ) );
    
    wxThumbnailItem::SetFilename( getName() );
    
    return true;
}

///////////////////////////////////////////////////////////////////////////////

wxSize UiGridItem::DoGetBestSize() const
{
    wxSize size;
    size.x = _thumbnail->GetWidth();
    size.y = _thumbnail->GetHeight();
    
    return size;
}

///////////////////////////////////////////////////////////////////////////////

bool UiGridItem::Draw(
    wxDC&            dc,
    wxThumbnailCtrl* WXUNUSED( ctrl ),
    const wxRect&    rect,
    int              WXUNUSED( style )
)
{
    if ( _thumbnail->Ok() ) {
        int x = rect.x + ( rect.width - _thumbnail->GetWidth() ) / 2;
        int y = rect.y + ( rect.height - _thumbnail->GetHeight() ) / 2;
        dc.DrawBitmap( *_thumbnail, x, y );
    }
    return true;
}

///////////////////////////////////////////////////////////////////////////////

bool UiGridItem::Load(
    wxThumbnailCtrl* ctrl,
    bool             forceLoad
)
{
    if ( _thumbnail->Ok() && !forceLoad )
        return true;
    
    if ( !wxFileExists( getName() ) )
        return false;
    
    // load image -> image
    if( _image != 0) delete _image;
    _image = new wxImage( getName() );

    // scale image -> thumbnail
    if ( _thumbnail != 0) delete _thumbnail;
    _thumbnail = new wxBitmap(
        _image->Scale( (int) ( _image->GetWidth() * _scale ),
                       (int) ( _image->GetHeight() * _scale ) ) );
    return true;
}

///////////////////////////////////////////////////////////////////////////////

void UiGridItem::setScale( float value )
{
    if ( _image == 0 )
        return;
    _scale = value;
    if ( _thumbnail != 0 ) {
        delete _thumbnail;
    }

    _thumbnail = new wxBitmap(
        _image->Scale( (int) ( _image->GetWidth() * _scale ),
                       (int) ( _image->GetHeight() * _scale ) ) );
}
