// 
//  uiSearchFrame.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiSearchFrame.h 2940 2008-04-15 03:48:29Z benoit $
// 

#ifndef _UISEARCHFRAME_H_
#define _UISEARCHFRAME_H_

#include <wx/frame.h>
#include <wx/toolbar.h>
#include <wx/spinctrl.h>

#include "uiGridItem.h"
#include "collectionReport.h"
#include "shared_ptr.h"
#include <set>

class wxSpinCtrl;
class wxThumbnailCtrl;
class wxThumbnailEvent;
class UiSearchOptions;

///////////////////////////////////////////////////////////////////////////////

enum {
    ID_FRAME,
    ID_MENU,
    ID_TOOLBAR1,
    ID_RESULTSSLIDER,
    ID_SCROLLEDWINDOW1,
    ID_SLIDER
};

///////////////////////////////////////////////////////////////////////////////

#define SYMBOL_ROOTFRAME_STYLE \
        wxDEFAULT_FRAME_STYLE | \
        wxCAPTION | \
        wxRESIZE_BORDER | \
        wxSYSTEM_MENU | \
        wxMINIMIZE | \
        wxMAXIMIZE | \
        wxMINIMIZE_BOX | \
        wxMAXIMIZE_BOX | \
        wxCLOSE_BOX
#define SYMBOL_ROOTFRAME_TITLE _T( "Image Browser" )

class UiSearchFrame: public wxFrame
{    
    DECLARE_CLASS( UiSearchFrame )

public:

    UiSearchFrame(
        wxWindow*       parent,
        wxWindowID      id      = ID_FRAME,
        const wxString& caption = SYMBOL_ROOTFRAME_TITLE,
        const wxPoint&  pos     = wxDefaultPosition,
        const wxSize&   size    = wxDefaultSize,
        long            style   = SYMBOL_ROOTFRAME_STYLE
    );

    void SetCollection(
        const shared_ptr< const CollectionReport::Header >& header,
        const shared_ptr< CollectionReport >&               collection
    );
    void CreateControls();

    void OnIdle( wxIdleEvent& event );
    void OnMenuFileExitClick( wxCommandEvent& event );
    void OnRandom( wxCommandEvent& event );

    void OnToolbarNew( wxCommandEvent& event );
    void OnToolbarLoad( wxCommandEvent& event );
    void OnToolbarRetrieveSimilar( wxCommandEvent& event );

    void OnZoom( wxCommandEvent& event );
    void OnNbResultsChanged( wxCommandEvent& event );

    static bool ShowToolTips();
    
    void displayGridResults();

    void OnItemSelect( wxThumbnailEvent& event );
    void OnItemDeSelect( wxThumbnailEvent& event );
    void OnItemClick( wxThumbnailEvent& event );
    void OnItemDoubleClick( wxThumbnailEvent& event );
    void OnItemRightClick( wxThumbnailEvent& event );
    void OnItemContextClick( wxContextMenuEvent& event );
 
    void ContextMenu( const wxPoint& pos = wxDefaultPosition );

private:

    void nbResultsUp();
    void nbResultsDown();
    void retrieveSimilar( UiGridItem* );

    wxSlider*        _resultCountSlider;
    wxThumbnailCtrl* _scrolledWindow;
    wxSlider*        _zoomSlider;
    UiSearchOptions* _searchOptions;

    float            _scaleFactor;
    UiGridItem*      _selectedItem;

    shared_ptr< CollectionReport > _collection;
    std::multiset< SearchResult >  _currentResults;

    DECLARE_EVENT_TABLE()
};

#endif /* _UISEARCHFRAME_H_ */
