// 
//  uiSearchOptions.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiSearchOptions.cpp 2940 2008-04-15 03:48:29Z benoit $
// 

#include "uiSearchOptions.h"
#include "histogramRanker.h"
#include "textureRanker.h"
#include "histogramFeature.h"
#include "textureFeature.h"

///////////////////////////////////////////////////////////////////////////////

enum {
    ID_FEATURE_STATICBOX,
    ID_DISTANCE_STATICBOX,
    
    ID_FEATURE_RADIO_RGBHISTO,
    ID_FEATURE_RADIO_HVCHISTO,
    ID_FEATURE_RADIO_COOCC,
    ID_FEATURE_RADIO_CORR,
    
    ID_DISTANCE_RADIO_EUCLIDEAN,
    ID_DISTANCE_RADIO_MANHATTAN,
    ID_DISTANCE_RADIO_WEIGHTED,
    ID_DISTANCE_RADIO_INTERSECTION
};

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( UiSearchOptions, wxPanel )
    EVT_RADIOBUTTON( ID_FEATURE_RADIO_RGBHISTO, \
                     UiSearchOptions::OnFeatureChange )
    EVT_RADIOBUTTON( ID_FEATURE_RADIO_HVCHISTO, \
                     UiSearchOptions::OnFeatureChange )
    EVT_RADIOBUTTON( ID_FEATURE_RADIO_COOCC, \
                     UiSearchOptions::OnFeatureChange )
    EVT_RADIOBUTTON( ID_FEATURE_RADIO_CORR, \
                     UiSearchOptions::OnFeatureChange )

    EVT_RADIOBUTTON( ID_DISTANCE_RADIO_EUCLIDEAN, \
                     UiSearchOptions::OnDistanceChange )
    EVT_RADIOBUTTON( ID_DISTANCE_RADIO_MANHATTAN, \
                     UiSearchOptions::OnDistanceChange )
    EVT_RADIOBUTTON( ID_DISTANCE_RADIO_WEIGHTED, \
                     UiSearchOptions::OnDistanceChange )
    EVT_RADIOBUTTON( ID_DISTANCE_RADIO_INTERSECTION, \
                     UiSearchOptions::OnDistanceChange )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

UiSearchOptions::UiSearchOptions( wxWindow* parent )
: wxPanel( parent ),
  _featureStaticBox(
        new wxStaticBox(
            this,
            ID_FEATURE_STATICBOX,
            _T( "Image feature" ) ) ),
  _distanceStaticBox(
        new wxStaticBox(
            this,
            ID_DISTANCE_STATICBOX,
            _T( "Distance type" ) ) ),
  _hasHistogramRGB( false ),
  _hasHistogramHVC( false ),
  _hasCoOccurence( false ),
  _hasCorrelogram( false ),
  _mode( NONE ),
  _header()
{
    wxBoxSizer* sizer = new wxBoxSizer( wxHORIZONTAL );

    wxStaticBoxSizer* featureBoxSizer =
        new wxStaticBoxSizer( _featureStaticBox, wxVERTICAL );

    wxArrayString featureStrings;
    featureStrings.Add( _T( "RGB Histogram" ) );
    featureStrings.Add( _T( "HVC Histogram" ) );
    featureStrings.Add( _T( "Co-Occurence Matrix Statistics" ) );
    featureStrings.Add( _T( "Color Correlogram Statistics" ) );

    for( int i = 0; i < 4; ++ i ) {
        _featureRadioButtons[i] =
            new wxRadioButton( this,
                               ID_FEATURE_RADIO_RGBHISTO + i,
                               featureStrings[i],
                               wxDefaultPosition,
                               wxDefaultSize,
                               ( i ? 0 : wxRB_GROUP ) );
        featureBoxSizer->Add( _featureRadioButtons[i] );
    }

    sizer->Add( featureBoxSizer, 1, wxGROW );

    wxStaticBoxSizer* distanceBoxSizer =
        new wxStaticBoxSizer( _distanceStaticBox, wxVERTICAL );

    wxArrayString distanceStrings;
    distanceStrings.Add( _T( "Euclidean" ) );
    distanceStrings.Add( _T( "Manhattan (L1)" ) );
    distanceStrings.Add( _T( "Weighted Euclidean" ) );
    distanceStrings.Add( _T( "Histogram Intersection" ) );

    for( int i = 0; i < 4; ++ i ) {
        _distanceRadioButtons[i] =
            new wxRadioButton( this,
                               ID_DISTANCE_RADIO_EUCLIDEAN + i,
                               distanceStrings[i],
                               wxDefaultPosition,
                               wxDefaultSize,
                               ( i ? 0 : wxRB_GROUP ) );
        distanceBoxSizer->Add( _distanceRadioButtons[i] );
    }

    sizer->Add( distanceBoxSizer, 1, wxGROW | wxLEFT, 20 );
    SetSizer( sizer );
}

///////////////////////////////////////////////////////////////////////////////

void UiSearchOptions::SetCollectionHeader(
    const shared_ptr< const CollectionReport::Header >& header
)
{
    _header = header;
    detectOptions();
    Enable();
}

///////////////////////////////////////////////////////////////////////////////

void UiSearchOptions::OnFeatureChange( wxCommandEvent& event )
{
    Mode newMode = NONE;

    if( _featureRadioButtons[0]->GetValue() ) {
        newMode = COLOR;
        _currentType = ImageFeature::HISTOGRAM;
        _currentColorSpace = ColorSpaces::RGB;
    }
    else if( _featureRadioButtons[1]->GetValue() ) {
        newMode = COLOR;
        _currentType = ImageFeature::HISTOGRAM;
        _currentColorSpace = ColorSpaces::HVC;
    }
    else if( _featureRadioButtons[2]->GetValue() ) {
        newMode = TEXTURE;
        _currentType = ImageFeature::CO_OCCURENCE;
        _currentColorSpace = ColorSpaces::Grayscale;
    }
    else if( _featureRadioButtons[3]->GetValue() ) {
        newMode = TEXTURE;
        _currentType = ImageFeature::CORRELOGRAM;
        _currentColorSpace = ColorSpaces::UNKNOWN;
    }

    // Switching from color to texture
    if( _mode == COLOR && newMode == TEXTURE  ) {
        // If the current distance is not supported
        // in texture search
        if( _distanceRadioButtons[2]->GetValue() ||
            _distanceRadioButtons[3]->GetValue() ) {

            // Force Euclidean distance
            _distanceRadioButtons[0]->SetValue( true );
            _currentDistance = ImageRanker::Euclidean;
        }
    }

    // Set the new mode
    _mode = newMode;

    Enable();
}

///////////////////////////////////////////////////////////////////////////////

void UiSearchOptions::OnDistanceChange( wxCommandEvent& event )
{
    if( _distanceRadioButtons[0]->GetValue() ) {
        _currentDistance = ImageRanker::Euclidean;
    }
    else if( _distanceRadioButtons[1]->GetValue() ) {
        _currentDistance = ImageRanker::Manhattan;
    }
    else if( _distanceRadioButtons[2]->GetValue() ) {
        _currentDistance = ImageRanker::Weighted;
    }
    else if( _distanceRadioButtons[3]->GetValue() ) {
        _currentDistance = ImageRanker::Intersection;
    }
}

///////////////////////////////////////////////////////////////////////////////

bool UiSearchOptions::Enable( bool enable )
{
    bool returnVal = false;
    returnVal |= _featureRadioButtons[0]->Enable( enable && _hasHistogramRGB );
    returnVal |= _featureRadioButtons[1]->Enable( enable && _hasHistogramHVC );
    returnVal |= _featureRadioButtons[2]->Enable( enable && _hasCoOccurence );
    returnVal |= _featureRadioButtons[3]->Enable( enable && _hasCorrelogram );
    returnVal |= _distanceRadioButtons[0]->Enable( enable && ( _mode != NONE ) );
    returnVal |= _distanceRadioButtons[1]->Enable( enable && ( _mode != NONE ) );
    returnVal |= _distanceRadioButtons[2]->Enable( enable && ( _mode == COLOR ) );
    returnVal |= _distanceRadioButtons[3]->Enable( enable && ( _mode == COLOR ) );
    return returnVal;
}

///////////////////////////////////////////////////////////////////////////////

bool UiSearchOptions::Disable()
{
    return Enable( false );
}

///////////////////////////////////////////////////////////////////////////////

void UiSearchOptions::detectOptions()
{
    if( !_header ) {
        _hasHistogramRGB = false;
        _hasHistogramHVC = false;
        _hasCoOccurence = false;
        _hasCorrelogram = false;
        _mode = NONE;
        return;
    }

    // Find which features are available
    _hasHistogramRGB =
        _header->getFeature( ImageFeature::HISTOGRAM, ColorSpaces::RGB ).second;
    _hasHistogramHVC =
        _header->getFeature( ImageFeature::HISTOGRAM, ColorSpaces::HVC ).second;
    _hasCoOccurence = 
        _header->getFeature( ImageFeature::CO_OCCURENCE ).second;
    _hasCorrelogram =
        _header->getFeature( ImageFeature::CORRELOGRAM ).second;

    if( _hasHistogramRGB ) {
        _featureRadioButtons[0]->SetValue( true );
        _currentType = ImageFeature::HISTOGRAM;
        _currentColorSpace = ColorSpaces::RGB;
        _mode = COLOR;
    }
    else if( _hasHistogramHVC ) {
        _featureRadioButtons[1]->SetValue( true );
        _currentType = ImageFeature::HISTOGRAM;
        _currentColorSpace = ColorSpaces::HVC;
        _mode = COLOR;
    }
    else if( _hasCoOccurence ) {
        _featureRadioButtons[2]->SetValue( true );
        _currentType = ImageFeature::CO_OCCURENCE;
        _currentColorSpace = ColorSpaces::Grayscale;
        _mode = TEXTURE;
    }
    else if( _hasCorrelogram ) {
        _featureRadioButtons[3]->SetValue( true );
        _currentType = ImageFeature::CORRELOGRAM;
        _currentColorSpace = ColorSpaces::UNKNOWN;
        _mode = TEXTURE;
    }
    _distanceRadioButtons[0]->SetValue( true );
    _currentDistance = ImageRanker::Euclidean;
}

///////////////////////////////////////////////////////////////////////////////

shared_ptr< ImageRanker > UiSearchOptions::generateImageRanker(
    shared_ptr< const ImageReport > query
)
{
    shared_ptr< ImageRanker > ranker;

    switch( _currentType ) {
        case ImageFeature::HISTOGRAM:
        {
            ranker.reset( new HistogramRanker(
                _currentDistance,
                dynamic_pointer_cast< const HistogramFeature >(
                query->getFeature( _currentType, _currentColorSpace ) ) )
            );
        } break;
        case ImageFeature::CO_OCCURENCE:
        case ImageFeature::CORRELOGRAM:
        {
            ranker.reset( new TextureRanker(
                _currentDistance,
                dynamic_pointer_cast< const TextureFeature >(
                    query->getFeature( _currentType ) ) )
            );
            
        } break;
    }

    return ranker;
}
