// 
//  uiSplitHVC.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiSplitHVC.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "uiSplitHVC.h"
#include "colorSpaces.h"
#include <wx/spinctrl.h>

///////////////////////////////////////////////////////////////////////////////

enum {
    ID_SPLIT_R,
    ID_SPLIT_G,
    ID_SPLIT_B,
    ID_SPLIT_HVC
};

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( UiSplitHVC, wxPanel )
    EVT_SPINCTRL( ID_SPLIT_R, UiSplitHVC::OnSplitChange )
    EVT_SPINCTRL( ID_SPLIT_G, UiSplitHVC::OnSplitChange )
    EVT_SPINCTRL( ID_SPLIT_B, UiSplitHVC::OnSplitChange )
    EVT_SPINCTRL( ID_SPLIT_HVC, UiSplitHVC::OnSplitChange )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

int64_t UiSplitHVC::Values::toParams() const {
    assert( sizeof( int64_t ) == 8 );
    assert( sizeof( int ) == 4 );

    //  val: |r|g|b|n   |d|
    // byte: |0|1|2|3456|7|
    unsigned char params[8];    
    params[0] = static_cast< unsigned char >( _rSplit );
    params[1] = static_cast< unsigned char >( _gSplit );
    params[2] = static_cast< unsigned char >( _bSplit );
    *( reinterpret_cast< unsigned int* >( params+3 ) ) = _nbBins;

    return *( reinterpret_cast< int64_t* >( params ) );
}

///////////////////////////////////////////////////////////////////////////////

UiSplitHVC::UiSplitHVC( wxWindow* parent )
: wxPanel( parent ),
  _hvcStaticBox( new wxStaticBox(
                  this,
                  wxID_STATIC,
                  _T("k-Means color space reduction") ) ),
  _splitR( new wxSpinCtrl(
                    this,
                    ID_SPLIT_R,
                    wxT("3"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitG( new wxSpinCtrl(
                    this,
                    ID_SPLIT_G,
                    wxT("3"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitB( new wxSpinCtrl(
                    this,
                    ID_SPLIT_B,
                    wxT("3"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitHVC( new wxSpinCtrl(
                    this,
                    ID_SPLIT_HVC,
                    wxT("27"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitRtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "R" ) ) ),
  _splitGtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "G" ) ) ),
  _splitBtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "B" ) ) ),
  _splitHVCtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "HVC bins" ) ) )
{
    wxStaticBoxSizer* hvcStaticBoxSizer(
      new wxStaticBoxSizer( _hvcStaticBox, wxHORIZONTAL) );

    // R
    hvcStaticBoxSizer->Add(
        _splitRtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
    hvcStaticBoxSizer->Add( _splitR  );

    // G
    hvcStaticBoxSizer->Add(
        _splitGtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
    hvcStaticBoxSizer->Add( _splitG  );

    // B
    hvcStaticBoxSizer->Add(
        _splitBtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
    hvcStaticBoxSizer->Add( _splitB  );

    // HVC
    hvcStaticBoxSizer->Add(
        _splitHVCtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5
    );
    hvcStaticBoxSizer->Add( _splitHVC  );

    SetSizer( hvcStaticBoxSizer );
}

///////////////////////////////////////////////////////////////////////////////

bool UiSplitHVC::Enable( bool enable )
{
    bool returnVal = false;

    returnVal |= _hvcStaticBox->Enable( enable );
    returnVal |= _splitR->Enable( enable );
    returnVal |= _splitG->Enable( enable );
    returnVal |= _splitB->Enable( enable );
    returnVal |= _splitHVC->Enable( enable);

    returnVal |= _splitRtxt->Enable( enable );
    returnVal |= _splitGtxt->Enable( enable );
    returnVal |= _splitBtxt->Enable( enable );
    returnVal |= _splitHVCtxt->Enable( enable );

    return returnVal;
}

///////////////////////////////////////////////////////////////////////////////

bool UiSplitHVC::Disable()
{
    return Enable( false );
}

///////////////////////////////////////////////////////////////////////////////

void UiSplitHVC::OnSplitChange( wxSpinEvent &event )
{
    int n = _splitR->GetValue() * _splitG->GetValue() * _splitB->GetValue();

    if( _splitHVC->GetValue() > n )
        _splitHVC->SetValue( n );
}

///////////////////////////////////////////////////////////////////////////////

UiSplitHVC::Values UiSplitHVC::getValues() const
{
    Values values = { _splitR->GetValue(),
                      _splitG->GetValue(),
                      _splitB->GetValue(),
                      _splitHVC->GetValue() };
    return values;
}
