// 
//  uiSplitRGB.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiSplitRGB.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "uiSplitRGB.h"
#include "colorSpaces.h"

///////////////////////////////////////////////////////////////////////////////

enum {
    ID_SPLIT_R,
    ID_SPLIT_G,
    ID_SPLIT_B
};

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( UiSplitRGB, wxPanel )
    EVT_SPINCTRL( ID_SPLIT_R, UiSplitRGB::OnSplitChange )
    EVT_SPINCTRL( ID_SPLIT_G, UiSplitRGB::OnSplitChange )
    EVT_SPINCTRL( ID_SPLIT_B, UiSplitRGB::OnSplitChange )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

unsigned int UiSplitRGB::Values::nbBins() const
{
    return ( _rSplit * _gSplit * _bSplit );
}

///////////////////////////////////////////////////////////////////////////////

int64_t UiSplitRGB::Values::toParams() const {
    assert( sizeof( int64_t ) == 8 );
    assert( sizeof( int ) == 4 );

    //  val: |r|g|b|n   |d|
    // byte: |0|1|2|3456|7|
    unsigned char params[8];
    params[0] = static_cast< unsigned char >( _rSplit );
    params[1] = static_cast< unsigned char >( _gSplit );
    params[2] = static_cast< unsigned char >( _bSplit );
    *( reinterpret_cast< unsigned int* >( params+3 ) ) = nbBins();

    return *( reinterpret_cast< int64_t* >( params ) );
}

///////////////////////////////////////////////////////////////////////////////

UiSplitRGB::UiSplitRGB( wxWindow* parent )
: wxPanel( parent ),
  _rgbStaticBox( new wxStaticBox(
                    this,
                    wxID_STATIC,
                    _T("Static color space reduction") ) ),
  _splitR( new wxSpinCtrl(
                    this,
                    ID_SPLIT_R,
                    wxT("3"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitG( new wxSpinCtrl(
                    this,
                    ID_SPLIT_G,
                    wxT("3"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitB( new wxSpinCtrl(
                    this,
                    ID_SPLIT_B,
                    wxT("3"),
                    wxDefaultPosition,
                    wxDefaultSize,
                    wxSP_ARROW_KEYS,
                    1, 256,1 ) ),
  _splitRtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "R" ) ) ),
  _splitGtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "G" ) ) ),
  _splitBtxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "B" ) ) ),
  _nbBinsTxt( new wxStaticText(
                    this,
                    wxID_STATIC,
                    _T( "27 bins" ) ) )
{
    wxStaticBoxSizer* rgbStaticBoxSizer(
      new wxStaticBoxSizer( _rgbStaticBox, wxHORIZONTAL) );

    // R
    rgbStaticBoxSizer->Add(
        _splitRtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
    rgbStaticBoxSizer->Add( _splitR  );

    // G
    rgbStaticBoxSizer->Add(
        _splitGtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
    rgbStaticBoxSizer->Add( _splitG  );

    // B
    rgbStaticBoxSizer->Add(
        _splitBtxt, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
    rgbStaticBoxSizer->Add( _splitB  );

    // Nb bins
    rgbStaticBoxSizer->Add(
        _nbBinsTxt,
        0,
        wxALIGN_CENTER_VERTICAL | wxLEFT,
        20
    );

    SetSizer( rgbStaticBoxSizer );
}

///////////////////////////////////////////////////////////////////////////////

bool UiSplitRGB::Enable( bool enable )
{
    bool returnVal = false;

    returnVal |= _rgbStaticBox->Enable( enable );

    returnVal |= _splitR->Enable( enable );
    returnVal |= _splitG->Enable( enable );
    returnVal |= _splitB->Enable( enable );

    returnVal |= _splitRtxt->Enable( enable );
    returnVal |= _splitGtxt->Enable( enable );
    returnVal |= _splitBtxt->Enable( enable );

    returnVal |= _nbBinsTxt->Enable( enable );

    return returnVal;
}

///////////////////////////////////////////////////////////////////////////////

bool UiSplitRGB::Disable()
{
    return Enable( false );
}

///////////////////////////////////////////////////////////////////////////////

void UiSplitRGB::OnSplitChange( wxSpinEvent &event )
{
    wxString nbBinsString;
    int n = _splitR->GetValue() * _splitG->GetValue() * _splitB->GetValue();
    nbBinsString << n << _T( " bins" );

    _nbBinsTxt->SetLabel( nbBinsString  );
}

///////////////////////////////////////////////////////////////////////////////

UiSplitRGB::Values UiSplitRGB::getValues() const
{
    Values values = { _splitR->GetValue(),
                      _splitG->GetValue(),
                      _splitB->GetValue() };
    return values;
}
