// 
//  uiTextureTab.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: uiTextureTab.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "uiTextureTab.h"
#include "uiSplitRGB.h"
#include "uiSplitHVC.h"
#include "uiDistancesSelector.h"

///////////////////////////////////////////////////////////////////////////////

enum {
    ID_CHECK_COOCC,
    ID_CHECK_CORR,
    ID_RADIO_USE_RGB,
    ID_RADIO_USE_HVC
};

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( UiTextureTab, wxPanel )
    EVT_CHECKBOX( ID_CHECK_COOCC, UiTextureTab::OnGenerateCoOcc )
    EVT_CHECKBOX( ID_CHECK_CORR, UiTextureTab::OnGenerateCorr )
    EVT_RADIOBUTTON( ID_RADIO_USE_RGB, UiTextureTab::OnUseRGB )
    EVT_RADIOBUTTON( ID_RADIO_USE_HVC, UiTextureTab::OnUseHVC )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

UiTextureTab::UiTextureTab( wxWindow* parent )
: wxPanel( parent ),
  _generateCoOccurence( new wxCheckBox(
          this,
          ID_CHECK_COOCC,
          _T( "Generate co-occurence matrices" ) ) ),
  _generateCorrelogram( new wxCheckBox(
          this,
          ID_CHECK_CORR,
          _T( "Generate color correlograms" ) ) ),
  _useRGB( new wxRadioButton( this, ID_RADIO_USE_RGB,
                      _T( "Use RGB colorspace" ),
                      wxDefaultPosition,
                      wxDefaultSize,
                      wxRB_GROUP ) ),
  _useHVC( new wxRadioButton( this, ID_RADIO_USE_HVC,
                      _T( "Use HVC colorspace" ),
                      wxDefaultPosition,
                      wxDefaultSize ) ),
  _uiSplitRGB( new UiSplitRGB( this ) ),
  _uiSplitHVC( new UiSplitHVC( this ) ),
  _coOccDistances( new UiDistancesSelector( this ) ),
  _corrDistances( new UiDistancesSelector( this ) )
{
    // Vertical layout
    wxBoxSizer* topSizer = new wxBoxSizer( wxVERTICAL );

    const int spacing( 10 );

    topSizer->Add( _generateCoOccurence, 0, wxALL, spacing );

    // Co-Occurence options
    topSizer->Add( _coOccDistances, 0, wxLEFT, 3*spacing );
    _coOccDistances->Disable();

    topSizer->Add( _generateCorrelogram, 0, wxALL, spacing );

    // Correlogram options
    wxBoxSizer* correlogramSizer = new wxBoxSizer( wxVERTICAL );

    topSizer->Add( _corrDistances, 0, wxLEFT | wxBOTTOM, 3*spacing );
    _corrDistances->Disable();

    correlogramSizer->Add( _useRGB );
    _useRGB->SetValue( true );
    _useRGB->Disable();
    correlogramSizer->Add( _uiSplitRGB, 0, wxALL, spacing );
    _uiSplitRGB->Disable();

    correlogramSizer->Add( _useHVC );
    _useHVC->Disable();
    correlogramSizer->Add( _uiSplitHVC , 0, wxALL, spacing );
    _uiSplitHVC->Disable();

    topSizer->Add( correlogramSizer, 0, wxLEFT, 3*spacing );

    // Set layout
    SetSizer( topSizer );
}

///////////////////////////////////////////////////////////////////////////////

void UiTextureTab::OnGenerateCorr( wxCommandEvent& event )
{
    bool enable = _generateCorrelogram->GetValue();

    _useRGB->Enable( enable );
    _useHVC->Enable( enable );
    _corrDistances->Enable( enable );

    _uiSplitRGB->Enable( enable && _useRGB->GetValue() );
    _uiSplitHVC->Enable( enable && _useHVC->GetValue() );
}

///////////////////////////////////////////////////////////////////////////////

void UiTextureTab::OnGenerateCoOcc( wxCommandEvent& event )
{
    _coOccDistances->Enable( _generateCoOccurence->GetValue() );
}

///////////////////////////////////////////////////////////////////////////////

void UiTextureTab::OnUseRGB( wxCommandEvent& event )
{
    _uiSplitRGB->Enable();
    _uiSplitHVC->Disable();
}

///////////////////////////////////////////////////////////////////////////////

void UiTextureTab::OnUseHVC( wxCommandEvent& event )
{
    _uiSplitRGB->Disable();
    _uiSplitHVC->Enable();
}

///////////////////////////////////////////////////////////////////////////////

UiTextureTab::Values UiTextureTab::getValues() const
{
    Values vals;

    vals._generateCoOcc = _generateCoOccurence->GetValue();
    vals._distCoOcc = _coOccDistances->toParams();

    vals._generateCorr = _generateCorrelogram->GetValue();
    vals._distCorr = _corrDistances->toParams();

    vals._useRGB = _useRGB->GetValue();
    vals._rgbValues = _uiSplitRGB->getValues();

    vals._useHVC = _useHVC->GetValue();
    vals._hvcValues = _uiSplitHVC->getValues();
    
    return vals;
}
