// 
//  wxImageFile.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: wxImageFile.cpp 2928 2008-04-13 22:48:01Z benoit $
// 

#include "wxImageFile.h"
#include "image.h"
#include <string.h>
#include <stdlib.h>

#include <wx/image.h>

///////////////////////////////////////////////////////////////////////////////

wxImageFile::wxImageFile( const std::wstring& filePath )
: ImageFile( filePath )
{}

///////////////////////////////////////////////////////////////////////////////

wxImageFile::~wxImageFile()
{}

///////////////////////////////////////////////////////////////////////////////

ImageBase* wxImageFile::read() const
{
    wxString filePath( getFilePath().c_str() );
    wxImage wxImg( filePath );

    if( !wxImg.Ok() ) {
        throw "Unable to read file";
    }

    const unsigned int w( wxImg.GetWidth() );
    const unsigned int h( wxImg.GetHeight() );

    Image< STUInt8 >* img( dynamic_cast< Image< STUInt8 >* >(
        ImageBase::create( w, h, ColorSpaces::RGB, STUInt8 ) ) );

    // Copy the data
    memcpy( img->getPixels(), wxImg.GetData(), w*h*3 );

    return img;
}

///////////////////////////////////////////////////////////////////////////////

void wxImageFile::write( const ImageBase* image )
{
    const Image< STUInt8 >* img(
        dynamic_cast< const Image< STUInt8 >* >( image ) );
    assert( img != 0 );
    assert( img->getColorSpace() == ColorSpaces::RGB );

    // Get image size
    const unsigned int w( img->getWidth() );
    const unsigned int h( img->getHeight() );

    // Create a buffer for wxImage
    unsigned char* buffer = (unsigned char*) malloc( w*h*3 );

    // Copy image data to it
    memcpy( buffer, img->getPixels(), w*h*3 );

    // Create a wxImage using that buffer.
    // note: wxImage will free the allocated memory
    wxImage wxImg( img->getWidth(), img->getHeight(), buffer );

    wxString path( getFilePath().c_str() );
    if( !wxImg.SaveFile( path ) ) {
        throw "Unable to save file";
    }
}
