// 
//  wxThumbnailCtrl.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: wxThumbnailCtrl.cpp 2927 2008-04-13 22:47:17Z benoit $
// 

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include "wxThumbnailCtrl.h"
#include "wxThumbnailItem.h"
#include "wxThumbnailEvent.h"

#include <wx/settings.h>
#include <wx/arrimpl.cpp>
#include <wx/image.h>
#include <wx/filename.h>
#include <wx/dcbuffer.h>

///////////////////////////////////////////////////////////////////////////////

WX_DEFINE_OBJARRAY( wxThumbnailItemArray );

///////////////////////////////////////////////////////////////////////////////

DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_ITEM_SELECTED )
DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_ITEM_DESELECTED )
DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_LEFT_CLICK )
DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_MIDDLE_CLICK )
DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_RIGHT_CLICK )
DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_LEFT_DCLICK )
DEFINE_EVENT_TYPE( wxEVT_COMMAND_THUMBNAIL_RETURN )

///////////////////////////////////////////////////////////////////////////////

IMPLEMENT_CLASS( wxThumbnailCtrl, wxScrolledWindow )

///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE( wxThumbnailCtrl, wxScrolledWindow )

    EVT_PAINT( wxThumbnailCtrl::OnPaint )
    EVT_ERASE_BACKGROUND( wxThumbnailCtrl::OnEraseBackground )

    EVT_LEFT_DOWN( wxThumbnailCtrl::OnLeftClick)
    EVT_RIGHT_DOWN( wxThumbnailCtrl::OnRightClick)
    EVT_MIDDLE_DOWN( wxThumbnailCtrl::OnMiddleClick)
    EVT_LEFT_DCLICK( wxThumbnailCtrl::OnLeftDClick)
    EVT_CHAR( wxThumbnailCtrl::OnChar )
    EVT_SIZE( wxThumbnailCtrl::OnSize )
    EVT_SET_FOCUS( wxThumbnailCtrl::OnSetFocus)
    EVT_KILL_FOCUS( wxThumbnailCtrl::OnKillFocus)

    EVT_MENU( wxID_SELECTALL, wxThumbnailCtrl::OnSelectAll )
    EVT_UPDATE_UI( wxID_SELECTALL, wxThumbnailCtrl::OnUpdateSelectAll )
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////

wxThumbnailCtrl* wxThumbnailCtrl::s_currentThumbnailCtrl = NULL;

///////////////////////////////////////////////////////////////////////////////

wxThumbnailCtrl::wxThumbnailCtrl( )
{
    Init();
}

///////////////////////////////////////////////////////////////////////////////

wxThumbnailCtrl::wxThumbnailCtrl(
    wxWindow*      parent,
    wxWindowID     id,
    const wxPoint& pos,
    const wxSize&  size,
    long           style
)
{
    Init();
    Create( parent, id, pos, size, style );
}

///////////////////////////////////////////////////////////////////////////////

bool wxThumbnailCtrl::Create(
    wxWindow*      parent,
    wxWindowID     id,
    const wxPoint& pos,
    const wxSize&  size,
    long           style
)
{
    if ( !wxScrolledWindow::Create(
            parent, id, pos, size, style |wxFULL_REPAINT_ON_RESIZE ) ) {
        return false;
    }

    if ( !GetFont().Ok() ) {
        SetFont( wxSystemSettings::GetFont( wxSYS_DEFAULT_GUI_FONT ) );
    }

    CalculateOverallThumbnailSize();

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    
    SetBackgroundStyle( wxBG_STYLE_CUSTOM );

    // Tell the sizers to use the given or best size
    SetInitialSize( size );
    
    // Create a buffer
    RecreateBuffer(size);
    
    return true;
}

///////////////////////////////////////////////////////////////////////////////

// Member initialization
void wxThumbnailCtrl::Init()
{
    _thumbnailOverallSize = wxTHUMBNAIL_DEFAULT_OVERALL_SIZE;
    _thumbnailImageSize   = wxTHUMBNAIL_DEFAULT_IMAGE_SIZE;
    _freezeCount          = 0;
    _spacing              = wxTHUMBNAIL_DEFAULT_SPACING;
    _thumbnailMargin      = wxTHUMBNAIL_DEFAULT_MARGIN;
    _firstSelection       = -1;
    _lastSelection        = -1;
    _focussedThumbnailBackgroundColour
                          = wxTHUMBNAIL_DEFAULT_FOCUSSED_BACKGROUND;
    _unfocussedThumbnailBackgroundColour
                          = wxTHUMBNAIL_DEFAULT_UNFOCUSSED_BACKGROUND;
    _unselectedThumbnailBackgroundColour
                          = wxTHUMBNAIL_DEFAULT_UNSELECTED_BACKGROUND;
    _typeColour           = wxTHUMBNAIL_DEFAULT_TYPE_COLOUR;
    _tagColour            = wxTHUMBNAIL_DEFAULT_TAG_COLOUR;
    _focusRectColour      = wxTHUMBNAIL_DEFAULT_FOCUS_RECT_COLOUR;
    _focusItem = -1;
}

///////////////////////////////////////////////////////////////////////////////

/// Call Freeze to prevent refresh
void wxThumbnailCtrl::Freeze()
{
    _freezeCount ++;
}

///////////////////////////////////////////////////////////////////////////////

/// Call Thaw to refresh
void wxThumbnailCtrl::Thaw()
{
    _freezeCount--;

    if (_freezeCount == 0) {
        SetupScrollbars();
        Refresh();
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Append a single item
int wxThumbnailCtrl::Append( wxThumbnailItem* item )
{
    int sz = (int) GetCount();
    _items.Add( item );
    _firstSelection = -1;
    _lastSelection = -1;
    _focusItem = -1;

    item->Load( this, false );

    if ( _freezeCount == 0 ) {
        SetupScrollbars();
        Refresh();
    }
    return sz;
}

///////////////////////////////////////////////////////////////////////////////

/// Insert a single item
int wxThumbnailCtrl::Insert( wxThumbnailItem* item, int pos )
{
    _items.Insert( item, pos );
    _firstSelection = -1;
    _lastSelection = -1;
    _focusItem = -1;

    // Must now change selection indices because
    // items above it have moved up
    size_t i;
    for ( i = 0; i < _selections.GetCount(); i++ ) {
        if ( _selections[i] >= pos ) {
             _selections[i] = _selections[i] + 1;
        }
    }

    // Ditto for tags
    for (i = 0; i < _tags.GetCount(); i++) {
        if (_tags[i] >= pos) {
            _tags[i] = _tags[i] + 1;
        }
    }

    item->Load( this, false );

    if (_freezeCount == 0) {
        SetupScrollbars();
        Refresh();
    }

    return pos;
}

///////////////////////////////////////////////////////////////////////////////

/// Clear all items
void wxThumbnailCtrl::Clear()
{
    _firstSelection = -1;
    _lastSelection = -1;
    _focusItem = -1;
    _items.Clear();
    _selections.Clear();
    _tags.Clear();

    if ( _freezeCount == 0 ) {
        SetupScrollbars();
        Refresh();
    }
}

///////////////////////////////////////////////////////////////////////////////

static bool wxGetIntegerFromFilename( const wxString& fname, int& n )
{
    wxString filename( fname );
    wxStripExtension( filename );
    wxString strNum;

    size_t len = filename.Len();
    size_t i;
    for (i = len; i == 0; i--) {
        if ( wxIsdigit( filename[i] ) != 0 ) {
            strNum = filename[i] + strNum;
        }
        else {
            break;
        }
    }

    if ( strNum.IsEmpty() ) {
        return false;
    }
    else {
        n = wxAtoi( strNum );
        return true;
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Delete this item
void wxThumbnailCtrl::Delete(int n)
{
    if ( _firstSelection == n )
         _firstSelection = -1;
    if ( _lastSelection == n )
        _lastSelection = -1;
    if ( _focusItem == n )
        _focusItem = -1;

    if ( _selections.Index(n) != wxNOT_FOUND )
        _selections.Remove(n);

    if ( _tags.Index(n) != wxNOT_FOUND )
        _tags.Remove(n);

    _items.RemoveAt(n);

    // Must now change selection indices because
    // items have moved down
    size_t i;
    for (i = 0; i < _selections.GetCount(); i++) {
        if ( _selections[i] > n )
            _selections[i] = _selections[i] - 1;
    }

    if ( _freezeCount == 0 ) {
        SetupScrollbars();
        Refresh();
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Get the nth item
wxThumbnailItem* wxThumbnailCtrl::GetItem( int n )
{
    wxASSERT(n < GetCount());

    if (n < GetCount()) {
        return & _items[(size_t) n];
    }
    else {
        return NULL;
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Get the overall rect of the given item
bool wxThumbnailCtrl::GetItemRect(int n, wxRect& rect, bool transform)
{
    wxASSERT(n < GetCount());
    if (n < GetCount())
    {
        int row, col;
        if (!GetRowCol(n, GetClientSize(), row, col))
            return false;

        int x = col * (_thumbnailOverallSize.x + _spacing) + _spacing;
        int y = row * (_thumbnailOverallSize.y + _spacing) + _spacing;

        if ( transform )
        {
            int startX, startY;
            int xppu, yppu;
            GetScrollPixelsPerUnit(& xppu, & yppu);
            GetViewStart(& startX, & startY);
            x = x - startX*xppu;
            y = y - startY*yppu;
        }
        
        rect.x = x;
        rect.y = y;
        rect.width = _thumbnailOverallSize.x;
        rect.height = _thumbnailOverallSize.y;
        
        return true;
    }

    return false;
}

///////////////////////////////////////////////////////////////////////////////

/// Get the image rect of the given item
bool wxThumbnailCtrl::GetItemRectImage( int n, wxRect& rect, bool transform )
{
    wxASSERT(n < GetCount());

    wxRect outerRect;
    if (!GetItemRect(n, outerRect, transform))
        return false;

    rect.width = _thumbnailImageSize.x;
    rect.height = _thumbnailImageSize.y;
    rect.x = outerRect.x + (outerRect.width - rect.width)/2;
    rect.y = outerRect.y + (outerRect.height - rect.height)/2;
    if ( ( GetWindowStyle() & wxTH_EXTENSION_LABEL ) == 0 )
        rect.y -= _thumbnailTextHeight / 2;

    return true;
}

///////////////////////////////////////////////////////////////////////////////

/// The size of the image part
void wxThumbnailCtrl::SetThumbnailImageSize( const wxSize& sz )
{
    _thumbnailImageSize = sz;
    CalculateOverallThumbnailSize();

    if (GetCount() > 0 && _freezeCount == 0)
    {
        SetupScrollbars();
        Refresh();
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Calculate the outer thumbnail size based
/// on font used for text and inner size
void wxThumbnailCtrl::CalculateOverallThumbnailSize()
{
    wxCoord w;
    wxClientDC dc( this );
    dc.SetFont( GetFont() );
    dc.GetTextExtent( wxT( "X" ), &w, &_thumbnailTextHeight );

    // From left to right: margin, image, margin
    _thumbnailOverallSize.x = _thumbnailMargin * 2 + _thumbnailImageSize.x;

    // From top to bottom:
    // margin, text + margin (if wxTH_EXTENSION_LABEL set),
    // image, margin, text, margin
    if (GetWindowStyle() & wxTH_EXTENSION_LABEL) {
        _thumbnailOverallSize.y = _thumbnailMargin * 4
                                  + _thumbnailTextHeight * 2
                                  + _thumbnailImageSize.y;
    }
    else {
        _thumbnailOverallSize.y = _thumbnailMargin * 3
                                  + _thumbnailTextHeight
                                  + _thumbnailImageSize.y;
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Return the row and column given the client
/// size and a left-to-right, top-to-bottom layout
/// assumption
bool wxThumbnailCtrl::GetRowCol(
    int           item,
    const wxSize& clientSize,
    int&          row,
    int&          col
)
{
    wxASSERT(item < GetCount());
    if (item >= GetCount())
        return false;

    // How many can we fit in a row?

    int perRow = clientSize.x/(_thumbnailOverallSize.x + _spacing);
    if (perRow < 1)
        perRow = 1;

    row = item/perRow;
    col = item % perRow;

    return true;
}

///////////////////////////////////////////////////////////////////////////////

/// Select or deselect an item
void wxThumbnailCtrl::Select( int n, bool select )
{
    wxASSERT (n < GetCount());

    if (select) {
        if ( _selections.Index( n ) == wxNOT_FOUND ) {
            _selections.Add( n );
        }
    }
    else {
        if ( _selections.Index( n ) != wxNOT_FOUND ) {
            _selections.Remove( n );
        }
    }

    _firstSelection = n;
    _lastSelection = n;
    int oldFocusItem = _focusItem;
    _focusItem = n;

    if (_freezeCount == 0)
    {
        wxRect rect;
        GetItemRect(n, rect);
        RefreshRect(rect);

        if (oldFocusItem != -1 && oldFocusItem != n)
        {
            GetItemRect(oldFocusItem, rect);
            RefreshRect(rect);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Select or deselect a range
void wxThumbnailCtrl::SelectRange(int from, int to, bool select)
{
    int first = from;
    int last = to;
    if (first < last)
    {
        first = to;
        last = from;
    }
    wxASSERT (first >= 0 && first < GetCount());
    wxASSERT (last >= 0 && last < GetCount());

    Freeze();
    int i;
    for (i = first; i < last; i++)
    {
        Select(i, select);
    }
    _focusItem = to;
    Thaw();
}

///////////////////////////////////////////////////////////////////////////////

/// Select all
void wxThumbnailCtrl::SelectAll()
{
    Freeze();
    int i;
    for (i = 0; i < GetCount(); i++)
    {
        Select(i, true);
    }
    if (GetCount() > 0)
    {
        _focusItem = GetCount()-1;
    }
    else
    {
        _focusItem = -1;
    }
    Thaw();
}

///////////////////////////////////////////////////////////////////////////////

/// Select none
void wxThumbnailCtrl::SelectNone()
{
    Freeze();
    int i;
    for (i = 0; i < GetCount(); i++)
    {
        Select(i, false);
    }
    Thaw();
}

///////////////////////////////////////////////////////////////////////////////

/// Get the index of the single selection, if not multi-select.
/// Returns -1 if there is no selection.
int wxThumbnailCtrl::GetSelection() const
{
    if (_selections.GetCount() > 0)
        return _selections[0u];
    else
        return -1;
}

///////////////////////////////////////////////////////////////////////////////

/// Returns true if the item is selected
bool wxThumbnailCtrl::IsSelected(int n) const
{
    return (_selections.Index(n) != wxNOT_FOUND) ;
}

///////////////////////////////////////////////////////////////////////////////

/// Clears all selections
void wxThumbnailCtrl::ClearSelections()
{
    int count = GetCount();

    _selections.Clear();
    _firstSelection = -1;
    _lastSelection = -1;
    _focusItem = -1;

    if (count > 0 && _freezeCount == 0)
    {
        Refresh();
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Set the focus item
void wxThumbnailCtrl::SetFocusItem(int item)
{
    wxASSERT( item < GetCount() );
    if (item < GetCount()) {
        int oldFocusItem = _focusItem;
        _focusItem = item;
        
        if (_freezeCount == 0) {
            wxRect rect;
            if (oldFocusItem != -1) {
                GetItemRect(oldFocusItem, rect);
                RefreshRect(rect);
            }
            if (_focusItem != -1) {
                GetItemRect(_focusItem, rect);
                RefreshRect(rect);
            }
        }
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Tag or untag an item
void wxThumbnailCtrl::Tag(int n, bool tag)
{
    wxASSERT (n < GetCount());

    if ( tag ) {
        if ( _tags.Index(n) == wxNOT_FOUND )
            _tags.Add(n);
    }
    else {
        if ( _tags.Index(n) != wxNOT_FOUND )
             _tags.Remove(n);
    }

    if ( _freezeCount == 0 ) {
        wxRect rect;
        GetItemRect(n, rect);
        RefreshRect(rect);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Returns true if the item is tagged
bool wxThumbnailCtrl::IsTagged( int n ) const
{
    return ( _tags.Index( n ) != wxNOT_FOUND ) ;
}

///////////////////////////////////////////////////////////////////////////////

/// Clears all tags
void wxThumbnailCtrl::ClearTags()
{
    int count = GetCount();

    _tags.Clear();

    if (count > 0 && _freezeCount == 0)
    {
        Refresh();
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Painting
void wxThumbnailCtrl::OnPaint( wxPaintEvent& WXUNUSED( event ) )
{
    // Set this to 0 to compare it with the
    // unbuffered implementation
    wxBufferedPaintDC dc(this, _bufferBitmap);

    PrepareDC(dc);
 
    if ( _freezeCount > 0 )
        return;

    // Paint the background
    PaintBackground(dc);

    if ( GetCount() == 0 )
        return;

    wxRegion dirtyRegion = GetUpdateRegion();
    bool isFocussed = (FindFocus() == this);

    int i;
    int count = GetCount();
    int style = 0;
    wxRect rect, untransformedRect, imageRect, untransformedImageRect;
    for (i = 0; i < count; i++) {
        GetItemRect(i, rect);

        wxRegionContain c = dirtyRegion.Contains(rect);
        if ( c != wxOutRegion ) {
            GetItemRectImage(i, imageRect);
            style = 0;
            if ( IsSelected(i) )
                style |= wxTHUMBNAIL_SELECTED;
            if ( IsTagged(i) )
                style |= wxTHUMBNAIL_TAGGED;
            if ( isFocussed )
                style |= wxTHUMBNAIL_FOCUSSED;
            if ( isFocussed && i == _focusItem )
                style |= wxTHUMBNAIL_IS_FOCUS;

            GetItemRect(i, untransformedRect, false);
            GetItemRectImage(i, untransformedImageRect, false);
            
            DrawItemBackground( i, dc, untransformedRect,
                untransformedImageRect, style);
            DrawItem(i, dc, untransformedImageRect, style);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////

// Empty implementation, to prevent flicker
void wxThumbnailCtrl::OnEraseBackground( wxEraseEvent& WXUNUSED( event ) )
{}

///////////////////////////////////////////////////////////////////////////////

void wxThumbnailCtrl::OnSetFocus( wxFocusEvent& WXUNUSED( event ) )
{
    if ( GetCount() > 0 )
        Refresh();
}

///////////////////////////////////////////////////////////////////////////////

void wxThumbnailCtrl::OnKillFocus( wxFocusEvent& WXUNUSED( event ) )
{
    if ( GetCount() > 0 )
        Refresh();
}

///////////////////////////////////////////////////////////////////////////////

/// Left-click
void wxThumbnailCtrl::OnLeftClick( wxMouseEvent& event )
{
    SetFocus();
    int n;
    if ( HitTest(event.GetPosition(), n) ) {
        int flags = 0;
        if ( event.ControlDown() )
            flags |= wxTHUMBNAIL_CTRL_DOWN;
        if ( event.ShiftDown() )
            flags |= wxTHUMBNAIL_SHIFT_DOWN;
        if ( event.AltDown() )
            flags |= wxTHUMBNAIL_ALT_DOWN;

        EnsureVisible(n);
        DoSelection(n, flags);

        wxThumbnailEvent cmdEvent(
            wxEVT_COMMAND_THUMBNAIL_LEFT_CLICK,
            GetId());
        cmdEvent.SetEventObject(this);
        cmdEvent.SetIndex(n);
        cmdEvent.SetFlags(flags);
        GetEventHandler()->ProcessEvent(cmdEvent);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Right-click
void wxThumbnailCtrl::OnRightClick( wxMouseEvent& event )
{
    SetFocus();
    int n;
    if (HitTest(event.GetPosition(), n))
    {
        int flags = 0;
        if (event.ControlDown())
            flags |= wxTHUMBNAIL_CTRL_DOWN;
        if (event.ShiftDown())
            flags |= wxTHUMBNAIL_SHIFT_DOWN;
        if (event.AltDown())
            flags |= wxTHUMBNAIL_ALT_DOWN;
    
        // These two lines are replaced by two lines copied from the left mouse click:
        //if (_focusItem != n)
    //SetFocusItem(n);
    EnsureVisible(n);
 //copied from left mouse click
        DoSelection(n, flags);
//copied from left mouse click

        wxThumbnailEvent cmdEvent(
            wxEVT_COMMAND_THUMBNAIL_RIGHT_CLICK,
            GetId());
        cmdEvent.SetEventObject(this);
        cmdEvent.SetIndex(n);
        cmdEvent.SetFlags(flags);
        GetEventHandler()->ProcessEvent(cmdEvent);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Left-double-click
void wxThumbnailCtrl::OnLeftDClick( wxMouseEvent& event )
{
    int n;
    if (HitTest(event.GetPosition(), n))
    {
        int flags = 0;
        if (event.ControlDown())
            flags |= wxTHUMBNAIL_CTRL_DOWN;
        if (event.ShiftDown())
            flags |= wxTHUMBNAIL_SHIFT_DOWN;
        if (event.AltDown())
            flags |= wxTHUMBNAIL_ALT_DOWN;

        wxThumbnailEvent cmdEvent(
            wxEVT_COMMAND_THUMBNAIL_LEFT_DCLICK,
            GetId());
        cmdEvent.SetEventObject(this);
        cmdEvent.SetIndex(n);
        cmdEvent.SetFlags(flags);
        GetEventHandler()->ProcessEvent(cmdEvent);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Middle-click
void wxThumbnailCtrl::OnMiddleClick( wxMouseEvent& event )
{
    int n;
    if (HitTest(event.GetPosition(), n))
    {
        int flags = 0;
        if (event.ControlDown())
            flags |= wxTHUMBNAIL_CTRL_DOWN;
        if (event.ShiftDown())
            flags |= wxTHUMBNAIL_SHIFT_DOWN;
        if (event.AltDown())
            flags |= wxTHUMBNAIL_ALT_DOWN;

        wxThumbnailEvent cmdEvent(
            wxEVT_COMMAND_THUMBNAIL_MIDDLE_CLICK,
            GetId());
        cmdEvent.SetEventObject(this);
        cmdEvent.SetIndex(n);
        cmdEvent.SetFlags(flags);
        GetEventHandler()->ProcessEvent(cmdEvent);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Key press
void wxThumbnailCtrl::OnChar( wxKeyEvent& event )
{
    int flags = 0;
    if (event.ControlDown())
        flags |= wxTHUMBNAIL_CTRL_DOWN;
    if (event.ShiftDown())
        flags |= wxTHUMBNAIL_SHIFT_DOWN;
    if (event.AltDown())
        flags |= wxTHUMBNAIL_ALT_DOWN;

    if (event.GetKeyCode() == WXK_LEFT ||
        event.GetKeyCode() == WXK_RIGHT ||
        event.GetKeyCode() == WXK_UP ||
        event.GetKeyCode() == WXK_DOWN ||
        event.GetKeyCode() == WXK_HOME ||
        event.GetKeyCode() == WXK_PAGEUP ||
        event.GetKeyCode() == WXK_PAGEDOWN ||
        event.GetKeyCode() == WXK_PRIOR ||
        event.GetKeyCode() == WXK_NEXT ||
        event.GetKeyCode() == WXK_END)
    {
        Navigate(event.GetKeyCode(), flags);
    }
    else if (event.GetKeyCode() == WXK_RETURN)
    {
        wxThumbnailEvent cmdEvent(
            wxEVT_COMMAND_THUMBNAIL_RETURN,
            GetId());
        cmdEvent.SetEventObject(this);
        cmdEvent.SetFlags(flags);
        GetEventHandler()->ProcessEvent(cmdEvent);
    }
    else
        event.Skip();
}

///////////////////////////////////////////////////////////////////////////////

/// Keyboard navigation
bool wxThumbnailCtrl::Navigate( int keyCode, int flags )
{
    if (GetCount() == 0)
        return false;

    wxSize clientSize = GetClientSize();
    int perRow = clientSize.x/(_thumbnailOverallSize.x + _spacing);
    if (perRow < 1)
        perRow = 1;

    int rowsInView = clientSize.y/(_thumbnailOverallSize.y + _spacing);
    if (rowsInView < 1)
        rowsInView = 1;

    int focus = _focusItem;
    if (focus == -1)
        focus = _lastSelection;

    if (focus == -1 || focus >= GetCount())
    {
        _lastSelection = 0;
        DoSelection(_lastSelection, flags);
        ScrollIntoView(_lastSelection, keyCode);
        return true;
    }

    //FIXME Replace this with a switch case
    if ( keyCode == WXK_RIGHT ) {
        int next = focus + 1;
        if (next < GetCount())
        {
            DoSelection(next, flags);
            ScrollIntoView(next, keyCode);
        }
    }
    else if ( keyCode == WXK_LEFT ) {
        int next = focus - 1;
        if (next >= 0)
        {
            DoSelection(next, flags);
            ScrollIntoView(next, keyCode);
        }
    }
    else if (keyCode == WXK_UP)
    {
        int next = focus - perRow;
        if (next >= 0)
        {
            DoSelection(next, flags);
            ScrollIntoView(next, keyCode);
        }
    }
    else if ( keyCode == WXK_DOWN ) {
        int next = focus + perRow;
        if (next < GetCount())
        {
            DoSelection(next, flags);
            ScrollIntoView(next, keyCode);
        }
    }
    else if ( keyCode == WXK_PAGEUP || keyCode == WXK_PRIOR ) {
        int next = focus - (perRow * rowsInView);
        if (next < 0)
            next = 0;
        if (next >= 0)
        {
            DoSelection(next, flags);
            ScrollIntoView(next, keyCode);
        }
    }
    else if ( keyCode == WXK_PAGEDOWN || keyCode == WXK_NEXT ) {
        int next = focus + (perRow * rowsInView);
        if (next >= GetCount())
            next = GetCount() - 1;
        if (next < GetCount())
        {
            DoSelection(next, flags);
            ScrollIntoView(next, keyCode);
        }
    }
    else if ( keyCode == WXK_HOME ) {
        DoSelection(0, flags);
        ScrollIntoView(0, keyCode);
    }
    else if ( keyCode == WXK_END ) {
        DoSelection(GetCount()-1, flags);
        ScrollIntoView(GetCount()-1, keyCode);
    }
    return true;
}

///////////////////////////////////////////////////////////////////////////////

/// Scroll to see the image
void wxThumbnailCtrl::ScrollIntoView( int n, int keyCode )
{
    wxRect rect;
    GetItemRect(n, rect, false); // _Not_ relative to scroll start

    int ppuX, ppuY;
    GetScrollPixelsPerUnit(& ppuX, & ppuY);

    int startX, startY;
    GetViewStart(& startX, & startY);
    startX = 0;
    startY = startY * ppuY;

    int sx, sy;
    GetVirtualSize(& sx, & sy);
    sx = 0;
    if (ppuY != 0)
        sy = sy/ppuY;

    wxSize clientSize = GetClientSize();

    //FIXME Replace this with a switch case
    // Going down
    if ( keyCode == WXK_DOWN ||
         keyCode == WXK_RIGHT ||
         keyCode == WXK_END ||
         keyCode == WXK_NEXT ||
         keyCode == WXK_PAGEDOWN ) {
        if ( ( rect.y + rect.height ) > ( clientSize.y + startY ) ) {
            // Make it scroll so this item is at the bottom
            // of the window
            int y = rect.y 
                    -( clientSize.y - _thumbnailOverallSize.y - _spacing );
            SetScrollbars( ppuX, ppuY, sx, sy, 0, (int) (0.5 + y/ppuY) );
        }
        else if (rect.y < startY)
        {
            // Make it scroll so this item is at the top
            // of the window
            int y = rect.y ;
            SetScrollbars( ppuX, ppuY, sx, sy, 0, (int) ( 0.5 + y / ppuY ) );
        }
    }
    // Going up
    else if ( keyCode == WXK_UP ||
              keyCode == WXK_LEFT ||
              keyCode == WXK_HOME ||
              keyCode == WXK_PRIOR ||
              keyCode == WXK_PAGEUP ) {
        if (rect.y < startY) {
            // Make it scroll so this item is at the top
            // of the window
            int y = rect.y;
            SetScrollbars( ppuX, ppuY, sx, sy, 0, (int) ( 0.5 + y / ppuY ) );
        }
        else if ( ( rect.y + rect.height ) > ( clientSize.y + startY ) ) {
            // Make it scroll so this item is at the bottom
            // of the window
            int y = rect.y
                    - ( clientSize.y - _thumbnailOverallSize.y - _spacing );
            SetScrollbars( ppuX, ppuY, sx, sy, 0, (int) ( 0.5 + y/ppuY ) );
        }
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Scrolls the item into view if necessary
void wxThumbnailCtrl::EnsureVisible( int n )
{
    wxRect rect;
    GetItemRect(n, rect, false); // _Not_ relative to scroll start

    int ppuX, ppuY;
    GetScrollPixelsPerUnit(& ppuX, & ppuY);

    int startX, startY;
    GetViewStart(& startX, & startY);
    startX = 0;
    startY = startY * ppuY;

    int sx, sy;
    GetVirtualSize(& sx, & sy);
    sx = 0;
    if (ppuY != 0)
        sy = sy/ppuY;

    wxSize clientSize = GetClientSize();

    if ((rect.y + rect.height) > (clientSize.y + startY))
    {
        // Make it scroll so this item is at the bottom
        // of the window
        int y = rect.y - (clientSize.y - _thumbnailOverallSize.y - _spacing) ;
        SetScrollbars(ppuX, ppuY, sx, sy, 0, (int) (0.5 + y/ppuY));
    }
    else if (rect.y < startY)
    {
        // Make it scroll so this item is at the top
        // of the window
        int y = rect.y ;
        SetScrollbars(ppuX, ppuY, sx, sy, 0, (int) (0.5 + y/ppuY));
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Forces a reload of this item's thumbnail image
void wxThumbnailCtrl::Reload( int n )
{
    wxThumbnailItem* item = GetItem(n);
    if (item)
    {
        item->Load(this, true);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Sizing
void wxThumbnailCtrl::OnSize( wxSizeEvent& event )
{
    SetupScrollbars();
    RecreateBuffer();
    event.Skip();
}

///////////////////////////////////////////////////////////////////////////////

/// Set up scrollbars, e.g. after a resize
void wxThumbnailCtrl::SetupScrollbars()
{
    if (_freezeCount)
        return;

    if (GetCount() == 0)
    {
        SetScrollbars(0, 0, 0, 0, 0, 0);
        return;
    }

    int lastItem = wxMax(0, GetCount() - 1);
    int pixelsPerUnit = 10;
    wxSize clientSize = GetClientSize();

    int row, col;
    GetRowCol(lastItem, clientSize, row, col);

    int maxHeight = ( row + 1) * ( _thumbnailOverallSize.y + _spacing )
                    + _spacing;

    int unitsY = maxHeight/pixelsPerUnit;

    int startX, startY;
    GetViewStart(& startX, & startY);
    
    int maxPositionX = 0; // wxMax(sz.x - clientSize.x, 0);
    int maxPositionY = ( wxMax( maxHeight - clientSize.y, 0 ) ) / pixelsPerUnit;
    
    // Move to previous scroll position if
    // possible
    SetScrollbars(0, pixelsPerUnit,
        0, unitsY,
        wxMin(maxPositionX, startX), wxMin(maxPositionY, startY));
}

///////////////////////////////////////////////////////////////////////////////

/// Draws the item. Normally you override function in wxThumbnailItem.
bool wxThumbnailCtrl::DrawItem(
    int           n,
    wxDC&         dc,
    const wxRect& rect,
    int           style
)
{
    wxThumbnailItem* item = GetItem(n);
    if (item)
    {
        return item->Draw(dc, this, rect, style);
    }
    else
        return false;
}

///////////////////////////////////////////////////////////////////////////////

/// Draws the background for the item, including bevel
bool wxThumbnailCtrl::DrawItemBackground(
    int           n,
    wxDC&         dc,
    const wxRect& rect,
    const wxRect& imageRect,
    int           style
)
{
    wxThumbnailItem* item = GetItem( n );
    if ( item ) {
        return item->DrawBackground( dc, this, rect, imageRect, style, n );
    }
    else {
        return false;
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Do (de)selection
void wxThumbnailCtrl::DoSelection( int n, int flags )
{
    bool isSelected = IsSelected(n);

    wxArrayInt stateChanged;

    bool multiSelect = (GetWindowStyle() & wxTH_MULTIPLE_SELECT) != 0;

    if ( multiSelect &&
         ( flags & wxTHUMBNAIL_CTRL_DOWN ) == wxTHUMBNAIL_CTRL_DOWN ) {
        Select(n, !isSelected);
        stateChanged.Add(n);
    }
    else if ( multiSelect &&
              ( flags & wxTHUMBNAIL_SHIFT_DOWN ) == wxTHUMBNAIL_SHIFT_DOWN ) {
        // We need to find the last item selected,
        // and select all in between.

        int first = _firstSelection ;

        // Want to keep the 'first' selection
        // if we're extending the selection
        bool keepFirstSelection = false;
        wxArrayInt oldSelections = _selections;

        //TODO Need to refresh those that become unselected.
        //Store old selections, compare with new
        _selections.Clear(); 

        if ( _firstSelection != -1 &&
             _firstSelection < GetCount()
             && _firstSelection != n ) {
            int step = (n < _firstSelection) ? -1 : 1;
            int i;
            for (i = _firstSelection; i != n; i += step) {
                if (!IsSelected(i)) {
                    _selections.Add(i);
                    stateChanged.Add(i);

                    wxRect rect;
                    GetItemRect(i, rect);
                    RefreshRect(rect);
                }
            }
            keepFirstSelection = true;
        }

        // Refresh all the previously selected items that became unselected
        size_t i;
        for ( i = 0; i < oldSelections.GetCount(); i++ ) {
            if ( !IsSelected(oldSelections[i])) {
                wxRect rect;
                GetItemRect(oldSelections[i], rect);
                RefreshRect(rect);
            }
        }

        Select(n, true);
        if (stateChanged.Index(n) == wxNOT_FOUND)
            stateChanged.Add(n);

        if (keepFirstSelection)
            _firstSelection = first;
    }
    else {
        size_t i = 0;
        for (i = 0; i < _selections.GetCount(); i++)
        {
            wxRect rect;
            GetItemRect(_selections[i], rect);
            RefreshRect(rect);

            stateChanged.Add(i);
        }

        _selections.Clear();
        Select(n, true);
        if (stateChanged.Index(n) == wxNOT_FOUND)
            stateChanged.Add(n);
    }

    // Now notify the app of any selection changes
    size_t i = 0;
    for (i = 0; i < stateChanged.GetCount(); i++)
    {
        wxThumbnailEvent event( _selections.Index(
            stateChanged[i]) != wxNOT_FOUND ?
                                wxEVT_COMMAND_THUMBNAIL_ITEM_SELECTED :
                                wxEVT_COMMAND_THUMBNAIL_ITEM_DESELECTED,
            GetId()
        );
        event.SetEventObject(this);
        event.SetIndex(stateChanged[i]);
        GetEventHandler()->ProcessEvent(event);
    }
}

///////////////////////////////////////////////////////////////////////////////

/// Find the item under the given point
bool wxThumbnailCtrl::HitTest( const wxPoint& pt, int& n )
{
    wxSize clientSize = GetClientSize();
    int startX, startY;
    int ppuX, ppuY;
    GetViewStart(& startX, & startY);
    GetScrollPixelsPerUnit(& ppuX, & ppuY);

    int perRow = clientSize.x/(_thumbnailOverallSize.x + _spacing);
    if (perRow < 1)
        perRow = 1;

    int colPos = (int) ( pt.x / ( _thumbnailOverallSize.x + _spacing ) );
    int rowPos = (int) ( ( pt.y + startY * ppuY ) /
                         ( _thumbnailOverallSize.y + _spacing ) );

    int itemN = (rowPos * perRow + colPos);
    if ( itemN >= GetCount() )
        return false;

    wxRect rect;
    GetItemRect( itemN, rect );
    if ( rect.Contains( pt ) ) {
        n = itemN;
        return true;
    }

    return false;
}

///////////////////////////////////////////////////////////////////////////////

void wxThumbnailCtrl::OnSelectAll( wxCommandEvent& WXUNUSED( event ) )
{
    SelectAll();
}

///////////////////////////////////////////////////////////////////////////////

void wxThumbnailCtrl::OnUpdateSelectAll(wxUpdateUIEvent& event)
{
    event.Enable( GetCount() > 0 );
}    

///////////////////////////////////////////////////////////////////////////////

/// Paint the background
void wxThumbnailCtrl::PaintBackground( wxDC& dc )
{
    wxColour backgroundColour = GetBackgroundColour();
    if (!backgroundColour.Ok())
        backgroundColour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);

    // Clear the background
    dc.SetBrush(wxBrush(backgroundColour));
    dc.SetPen(*wxTRANSPARENT_PEN);
    wxRect windowRect(wxPoint(0, 0), GetClientSize());
    windowRect.x -= 2; windowRect.y -= 2;
    windowRect.width += 4; windowRect.height += 4;
    
    // We need to shift the rectangle to take into account
    // scrolling. Converting device to logical coordinates.
    CalcUnscrolledPosition(
        windowRect.x, windowRect.y, &windowRect.x, &windowRect.y );
    dc.DrawRectangle(windowRect);
}

///////////////////////////////////////////////////////////////////////////////

/// Recreate buffer bitmap if necessary
bool wxThumbnailCtrl::RecreateBuffer( const wxSize& size )
{
    wxSize sz = size;
    if (sz == wxDefaultSize)
        sz = GetClientSize();
        
    if (sz.x < 1 || sz.y < 1)
        return false;
        
    if ( !_bufferBitmap.Ok() ||
         _bufferBitmap.GetWidth() < sz.x ||
         _bufferBitmap.GetHeight() < sz.y ) {
            _bufferBitmap = wxBitmap(sz.x, sz.y);
    }

    return _bufferBitmap.Ok();
}

///////////////////////////////////////////////////////////////////////////////

wxSize wxThumbnailCtrl::DoGetBestSize() const
{
    wxSize sz = wxWindow::DoGetBestSize();
    return sz;
}
