// 
//  wxThumbnailCtrl.h
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: wxThumbnailCtrl.h 2927 2008-04-13 22:47:17Z benoit $
// 

#ifndef _WXTHUMBNAILCTRL_H_
#define _WXTHUMBNAILCTRL_H_

#include <wx/dynarray.h>
#include <wx/scrolwin.h>
#include <wx/bitmap.h>

/* Styles
 */

#define wxTH_MULTIPLE_SELECT    0x0010
#define wxTH_SINGLE_SELECT      0x0000
#define wxTH_TEXT_LABEL         0x0020
#define wxTH_IMAGE_LABEL        0x0040
#define wxTH_EXTENSION_LABEL    0x0080

/* Flags
 */

#define wxTHUMBNAIL_SHIFT_DOWN  0x01
#define wxTHUMBNAIL_CTRL_DOWN   0x02
#define wxTHUMBNAIL_ALT_DOWN    0x04

/* Defaults
 */

#define wxTHUMBNAIL_DEFAULT_OVERALL_SIZE wxSize(-1, -1)
#define wxTHUMBNAIL_DEFAULT_IMAGE_SIZE wxSize(200, 200)
#define wxTHUMBNAIL_DEFAULT_SPACING 3
#define wxTHUMBNAIL_DEFAULT_MARGIN 3
#define wxTHUMBNAIL_DEFAULT_UNFOCUSSED_BACKGROUND wxColour(175, 175, 175)
#define wxTHUMBNAIL_DEFAULT_FOCUSSED_BACKGROUND wxColour(140, 140, 140)
// #define wxTHUMBNAIL_DEFAULT_UNSELECTED_BACKGROUND wxColour(205, 205, 205)
#define wxTHUMBNAIL_DEFAULT_UNSELECTED_BACKGROUND wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE)
#define wxTHUMBNAIL_DEFAULT_TYPE_COLOUR wxColour(0, 0, 200)
#define wxTHUMBNAIL_DEFAULT_TAG_COLOUR wxColour(0, 0, 255)
#define wxTHUMBNAIL_DEFAULT_FOCUS_RECT_COLOUR wxColour(100, 80, 80)

///////////////////////////////////////////////////////////////////////////////

class wxThumbnailItem;
class wxThumbnailEvent;

WX_DECLARE_OBJARRAY(wxThumbnailItem, wxThumbnailItemArray);

class wxThumbnailCtrl: public wxScrolledWindow
{

    DECLARE_CLASS( wxThumbnailCtrl )
    DECLARE_EVENT_TABLE()

public:

    wxThumbnailCtrl();
    wxThumbnailCtrl(
        wxWindow*      parent,
        wxWindowID     id = -1,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize&  size = wxDefaultSize,
        long style = wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_LABEL
    );

    /// Creation
    bool Create(
        wxWindow*      parent,
        wxWindowID     id = -1,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize&  size = wxDefaultSize,
        long           style = wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_LABEL
    );

    /// Member initialisation
    void Init();

    /// Call Freeze to prevent refresh
    void Freeze();

    /// Call Thaw to refresh
    void Thaw();

    /// Scrolls the item into view if necessary
    void EnsureVisible(int n);

    /// Forces a reload of this item's thumbnail image
    void Reload(int n);

    /// Draws the item. Normally you override function in wxThumbnailItem.
    virtual bool DrawItem(int n, wxDC& dc, const wxRect& rect, int style) ;

    /// Draws the background for the item, including bevel
    virtual bool DrawItemBackground(
        int           n,
        wxDC&         dc,
        const wxRect& rect,
        const wxRect& imageRect,
        int           style
    );

    /// Append a single item
    virtual int Append( wxThumbnailItem* item );

    /// Insert a single item
    virtual int Insert( wxThumbnailItem* item, int pos = 0 );

    /// Clear all items
    virtual void Clear();

    /// Delete this item
    virtual void Delete( int n );

    /// Get the number of items in the control
    virtual int GetCount() const
    { return _items.GetCount(); }

    /// Is the control empty?
    bool IsEmpty() const
    { return GetCount() == 0; }

    /// Get the nth item
    wxThumbnailItem* GetItem( int n );

    /// Get the overall rect of the given item
    /// If transform is true, rect is relative to the scroll viewport
    /// (i.e. may be negative)
    bool GetItemRect( int item, wxRect& rect, bool transform = true );

    /// Get the image rect of the given item
    bool GetItemRectImage( int item, wxRect& rect, bool transform = true );

    /// Return the row and column given the client
    /// size and a left-to-right, top-to-bottom layout
    /// assumption
    bool GetRowCol( int item, const wxSize& clientSize, int& row, int& col );

    /// Get the focus item, or -1 if there is none
    int GetFocusItem() const
    { return _focusItem; }

    /// Set the focus item
    void SetFocusItem( int item );

    /// Select or deselect an item
    void Select( int n, bool select = true );

    /// Select or deselect a range
    void SelectRange( int from, int to, bool select = true );

    /// Tag or untag an item
    void Tag(int n, bool tag = true) ;

    /// Select all
    void SelectAll();

    /// Select none
    void SelectNone();

    /// Get the index of the single selection, if not multi-select.
    /// Returns -1 if there is no selection.
    int GetSelection() const;

    /// Get indexes of all selections, if multi-select
    const wxArrayInt& GetSelections() const
    { return _selections; }

    /// Get indexes of all tags
    const wxArrayInt& GetTags() const
    { return _tags; }

    /// Returns true if the item is selected
    bool IsSelected( int n ) const;

    /// Returns true if the item is tagged
    bool IsTagged(int n) const;

    /// Clears all selections
    void ClearSelections();

    /// Clears all tags
    void ClearTags();

// Visual properties

    /// The overall size of the thumbnail, including decorations.
    /// DON'T USE THIS from the application, since it will
    /// normally be calculated by SetThumbnailImageSize.
    void SetThumbnailOverallSize(const wxSize& sz)
    { _thumbnailOverallSize = sz; }
    const wxSize& GetThumbnailOverallSize() const
    { return _thumbnailOverallSize; }

    /// The size of the image part
    void SetThumbnailImageSize(const wxSize& sz);
    const wxSize& GetThumbnailImageSize() const
    { return _thumbnailImageSize; }

    /// The inter-item spacing
    void SetSpacing(int spacing)
    { _spacing = spacing; }

    int GetSpacing() const
    { return _spacing; }

    /// The margin between elements within the thumbnail
    void SetThumbnailMargin( int margin )
    { _thumbnailMargin = margin; }

    int GetThumbnailMargin() const
    { return _thumbnailMargin; }

    /// The height required for text in the thumbnail
    void SetThumbnailTextHeight( int h )
    { _thumbnailTextHeight = h; }

    int GetThumbnailTextHeight() const
    { return _thumbnailTextHeight; }

    /// The focussed and unfocussed background colour for a
    /// selected thumbnail
    void SetSelectedThumbnailBackgroundColour(
        const wxColour& focussedColour,
        const wxColour& unfocussedColour
    )
    {
        _focussedThumbnailBackgroundColour = focussedColour;
        _unfocussedThumbnailBackgroundColour = unfocussedColour;
    }

    const wxColour& GetSelectedThumbnailFocussedBackgroundColour() const
    { return _focussedThumbnailBackgroundColour; }

    const wxColour& GetSelectedThumbnailUnfocussedBackgroundColour() const
    { return _unfocussedThumbnailBackgroundColour; }

    /// The unselected background colour for a thumbnail
    void SetUnselectedThumbnailBackgroundColour(const wxColour& colour)
    { _unselectedThumbnailBackgroundColour = colour; }

    const wxColour& GetUnselectedThumbnailBackgroundColour() const
    { return _unselectedThumbnailBackgroundColour; }

    /// The colour for the type text (top left of thumbnail)
    void SetTypeColour( const wxColour& colour )
    { _typeColour = colour; }
    const wxColour& GetTypeColour() const
    { return _typeColour; }

    /// The colour for the tag outline
    void SetTagColour(const wxColour& colour)
    { _tagColour = colour; }
    const wxColour& GetTagColour() const
    { return _tagColour; }

    /// The focus rectangle pen colour
    void SetFocusRectColour( const wxColour& colour )
    { _focusRectColour = colour; }
    const wxColour& GetFocusRectColour() const
    { return _focusRectColour; }

    void OnSelectAll( wxCommandEvent& event );
    void OnUpdateSelectAll( wxUpdateUIEvent& event );
    
    /// Painting
    void OnPaint( wxPaintEvent& event );
    void OnEraseBackground( wxEraseEvent& event );

    /// Left-click
    void OnLeftClick( wxMouseEvent& event );

    /// Left-double-click
    void OnLeftDClick( wxMouseEvent& event );

    /// Middle-click
    void OnMiddleClick( wxMouseEvent& event );

    /// Right-click
    void OnRightClick( wxMouseEvent& event );

    /// Key press
    void OnChar( wxKeyEvent& event );

    /// Sizing
    void OnSize( wxSizeEvent& event );

    /// Setting/losing focus
    void OnSetFocus( wxFocusEvent& event );
    void OnKillFocus( wxFocusEvent& event );

    /// Set up scrollbars, e.g. after a resize
    void SetupScrollbars();

    /// Calculate the outer thumbnail size based
    /// on font used for text and inner size
    void CalculateOverallThumbnailSize();

    /// Do (de)selection
    void DoSelection( int n, int flags );

    /// Find the item under the given point
    bool HitTest( const wxPoint& pt, int& n );

    /// Keyboard navigation
    virtual bool Navigate( int keyCode, int flags );

    /// Scroll to see the image
    void ScrollIntoView( int n, int keyCode );

    /// Paint the background
    void PaintBackground( wxDC& dc );
    
    /// Recreate buffer bitmap if necessary
    bool RecreateBuffer( const wxSize& size = wxDefaultSize );

    static wxThumbnailCtrl* GetThumbnailCtrl()
    { return s_currentThumbnailCtrl; }
    
    wxSize DoGetBestSize() const ;

private:

    /// The items
    wxThumbnailItemArray    _items;

    /// The selections
    wxArrayInt              _selections;

    /// The tags
    wxArrayInt              _tags;

    /// Outer size of the thumbnail item
    wxSize                  _thumbnailOverallSize;

    /// Image size of the thumbnail item
    wxSize                  _thumbnailImageSize;

    /// The inter-item spacing
    int                     _spacing;

    /// The margin between the image/text and the edge of the thumbnail
    int                     _thumbnailMargin;

    /// The height of thumbnail text in the current font
    int                     _thumbnailTextHeight;

    /// Allows nested Freeze/Thaw
    int                     _freezeCount;

    /// First selection in a range
    int                     _firstSelection;

    /// Last selection
    int                     _lastSelection;

    /// Focus item
    int                     _focusItem;

    /// Current control, used in sorting
    static wxThumbnailCtrl* s_currentThumbnailCtrl;

    /// Focussed/unfocussed selected thumbnail background colours
    wxColour                _focussedThumbnailBackgroundColour;
    wxColour                _unfocussedThumbnailBackgroundColour;
    wxColour                _unselectedThumbnailBackgroundColour;
    wxColour                _focusRectColour;

    /// Type text colour
    wxColour                _typeColour;

    /// Tag colour
    wxColour                _tagColour;
    
    /// Buffer bitmap
    wxBitmap                _bufferBitmap;
};

/*!
 * wxThumbnailCtrl event macros
 */

BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_ITEM_SELECTED, 2600)
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_ITEM_DESELECTED, 2601)
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_LEFT_CLICK, 2602)
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_RIGHT_CLICK, 2603)
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_MIDDLE_CLICK, 2604)
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_LEFT_DCLICK, 2605)
    DECLARE_EVENT_TYPE(wxEVT_COMMAND_THUMBNAIL_RETURN, 2606)
END_DECLARE_EVENT_TYPES()

typedef void (wxEvtHandler::*wxThumbnailEventFunction)(wxThumbnailEvent&);

#define EVT_THUMBNAIL_ITEM_SELECTED(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_ITEM_SELECTED, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),
#define EVT_THUMBNAIL_ITEM_DESELECTED(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_ITEM_DESELECTED, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),
#define EVT_THUMBNAIL_LEFT_CLICK(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_LEFT_CLICK, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),
#define EVT_THUMBNAIL_RIGHT_CLICK(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_RIGHT_CLICK, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),
#define EVT_THUMBNAIL_MIDDLE_CLICK(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_MIDDLE_CLICK, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),
#define EVT_THUMBNAIL_LEFT_DCLICK(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_LEFT_DCLICK, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),
#define EVT_THUMBNAIL_RETURN(id, fn) DECLARE_EVENT_TABLE_ENTRY( wxEVT_COMMAND_THUMBNAIL_RETURN, id, -1, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( wxThumbnailEventFunction, & fn ), NULL ),

#endif /* _WXTHUMBNAILCTRL_H_ */
