// 
//  wxThumbnailItem.cpp
//  IFT630 - Processus concurrents et parallélisme
//  Benoit Gagnon (05 634 691)
//  
//  $Id: wxThumbnailItem.cpp 2926 2008-04-13 22:45:08Z benoit $
// 

#include "wxThumbnailItem.h"
#include "wxThumbnailCtrl.h"
#include <wx/colour.h>
#include <wx/filename.h>
#include <wx/pen.h>
#include <wx/dc.h>

///////////////////////////////////////////////////////////////////////////////

IMPLEMENT_CLASS( wxThumbnailItem, wxObject )

///////////////////////////////////////////////////////////////////////////////

/// Draw the item
bool wxThumbnailItem::Draw(
    wxDC&            WXUNUSED( dc ),
    wxThumbnailCtrl* WXUNUSED( ctrl ),
    const wxRect&    WXUNUSED( rect ),
    int              WXUNUSED( style )
)
{
    return false;
}

///////////////////////////////////////////////////////////////////////////////

/// Draw the item background
bool wxThumbnailItem::DrawBackground(
    wxDC&            dc,
    wxThumbnailCtrl* ctrl,
    const wxRect&    rect,
    const wxRect&    imageRect,
    int              style,
    int              WXUNUSED( index )
)
{
    wxColour mediumGrey = ctrl->GetUnselectedThumbnailBackgroundColour();
    wxColour unfocussedDarkGrey =
        ctrl->GetSelectedThumbnailUnfocussedBackgroundColour();
    wxColour focussedDarkGrey =
        ctrl->GetSelectedThumbnailFocussedBackgroundColour();
    wxColour darkGrey;
    if (style & wxTHUMBNAIL_FOCUSSED)
        darkGrey = focussedDarkGrey;
    else
        darkGrey = unfocussedDarkGrey;

    if (style & wxTHUMBNAIL_SELECTED) {
        wxBrush brush(darkGrey);
        wxPen pen(darkGrey);
        dc.SetBrush(brush);
        dc.SetPen(pen);
    }
    else {
        wxBrush brush(mediumGrey);
        wxPen pen(mediumGrey);
        dc.SetBrush(brush);
        dc.SetPen(pen);
    }

    wxString filename, ext;
    wxFileName::SplitPath( _filename, NULL, NULL, &filename, &ext );
    filename += wxChar( '.' );
    filename += ext;

    if ( !filename.IsEmpty() ) {
        dc.SetFont( ctrl->GetFont() );
        if (style & wxTHUMBNAIL_SELECTED)
            dc.SetTextForeground(*wxWHITE);
        else
            dc.SetTextForeground(*wxBLACK);
        dc.SetBackgroundMode(wxTRANSPARENT);

        int margin = ctrl->GetThumbnailMargin();

        wxRect fRect;
        fRect.x = rect.x + margin;
        fRect.y = rect.y
                  + rect.height
                  - ( rect.height - imageRect.height ) / 2;
        fRect.width = rect.width - 2 * margin;
        fRect.height = ( rect.height - imageRect.height ) / 2
                       + margin;

        wxCoord textW, textH;
        dc.GetTextExtent( filename, & textW, & textH );

        dc.SetClippingRegion(fRect);
        int x = fRect.x + wxMax(0, (fRect.width - textW)/2);
        int y = fRect.y - margin;
        dc.DrawText( filename, x, y );
        dc.DestroyClippingRegion();
    }
    if ( !filename.IsEmpty() ) {
        wxString file, name, ext;
        wxSplitPath(filename, & file, & name, & ext);
        if ( !ext.IsEmpty() &&
             ( ctrl->GetWindowStyle() & wxTH_EXTENSION_LABEL ) ) {
            
            ext.MakeUpper();
            dc.SetFont( ctrl->GetFont() );
            if ( style & wxTHUMBNAIL_SELECTED )
                dc.SetTextForeground(*wxWHITE);
            else
                dc.SetTextForeground( ctrl->GetTypeColour() );
            dc.SetBackgroundMode( wxTRANSPARENT );
            
            int margin = ctrl->GetThumbnailMargin();
            
            wxRect fRect;
            fRect.x = rect.x + margin;
            fRect.y = rect.y + margin;
            fRect.width = rect.width - 2*margin;
            fRect.height = (rect.height - imageRect.height)/2 - 2*margin;
            
            wxCoord textW, textH;
            dc.GetTextExtent(ext, & textW, & textH);
            
            dc.SetClippingRegion(fRect);
            int x = fRect.x ;
            int y = fRect.y ;
            dc.DrawText(ext, x, y);
            dc.DestroyClippingRegion();
        }
    }

    // If the item itself is the focus, draw a dotted
    // rectangle around it
    if ( style & wxTHUMBNAIL_IS_FOCUS ) {
        wxPen dottedPen(ctrl->GetFocusRectColour(), 1, wxDOT);
        dc.SetPen(dottedPen);
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        wxRect focusRect = imageRect;
        --focusRect.x;
        --focusRect.y;
        focusRect.width += 2;
        focusRect.height += 2;
        dc.DrawRectangle(focusRect);
    }
    
    return true;
}
